//
// C++ Implementation: kpgtypesfolder
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgtypesfolder.h"

#include <qtable.h>

#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "kpgconnection.h"
#include "kpgschema.h"
#include "kpgtype.h"
#include "../PropertyPages/kpgtypesfolderproppage.h"
#include "../kpgsqldialog.h"

KPGTypesFolder::KPGTypesFolder(KPGSchema *parent, pqxx::oid oidNameSpace)
 : KPGItemsFolder(parent, i18n("Types"))
{
	setPixmap(0, *m_pIconFolderRed);
	m_oidNameSpace = oidNameSpace;
}


KPGTypesFolder::~KPGTypesFolder()
{
}

bool KPGTypesFolder::refresh(pqxx::work &pqxxXaction)
{
  // delete all child items 
  while (QListViewItem * pItem = firstChild())
    delete pItem;

	// obtain list of data types
  QString strQuery("SELECT t.oid, t.typname, description, CASE WHEN t.typtype = 'b' THEN 'base' WHEN t.typtype = 'c' THEN 'composite' WHEN t.typtype = 'p' THEN 'pseudo' ELSE 'other' END as type, t.typlen, t.typbyval, t.typisdefined, t.typdelim, CASE WHEN t.typalign = 'c' THEN 'char' WHEN t.typalign = 's' THEN 'short' WHEN t.typalign = 'i' THEN 'int' WHEN t.typalign = 'd' THEN 'double' ELSE 'other' END AS align, CASE WHEN t.typstorage = 'p' THEN 'PLAIN' WHEN t.typstorage = 'e' THEN 'EXTERNAL' WHEN t.typstorage = 'm' THEN 'MAIN' WHEN t.typstorage = 'x' THEN 'EXTENDED' END AS typstorage, t.typelem, e.typname as element, t.typinput::oid AS typinputoid, t.typinput, t.typoutput::oid AS typoutputoid, t.typoutput, ct.oid AS tableoid, ct.relname AS tablename, pg_get_userbyid(t.typowner) as owner ");
	strQuery.append("FROM pg_type t ");
	strQuery.append("LEFT OUTER JOIN pg_type e ON e.oid=t.typelem ");
	strQuery.append("LEFT OUTER JOIN pg_class ct ON ct.oid=t.typrelid AND ct.relkind <> 'c' ");
	strQuery.append("LEFT OUTER JOIN pg_description des ON des.objoid=t.oid ");
	strQuery.append("WHERE t.typtype != 'd' AND t.typnamespace = " + QString("%1").arg(m_oidNameSpace) + " ");
	strQuery.append("ORDER BY typname"); // AND t.typname NOT LIKE '\\\\_%%' 
			
  try
  {
    m_pqxxResultTypes = pqxxXaction.Exec(strQuery.latin1());

		KPGType *pgType = 0;
		
    for (result::size_type i = 0; i != m_pqxxResultTypes.size(); ++i)
    {
			pqxx::oid oid;
			m_pqxxResultTypes[i][0].to(oid);
		
      if(pgType == 0)
				pgType = new KPGType(this, m_pqxxResultTypes[i]["typname"].c_str(), oid);
			else
				pgType = new KPGType(this, pgType, m_pqxxResultTypes[i]["typname"].c_str(), oid);
				
			int iTypLen; 
			bool bByVal; 
			bool bIsDefined; 
			pqxx::oid oidTypElem = 0;
			pqxx::oid oidInputFunc = 0;
			pqxx::oid oidOutputFunc = 0;
			pqxx::oid oidTable = 0;
			
			m_pqxxResultTypes[i]["typlen"].to(iTypLen);
			m_pqxxResultTypes[i]["typbyval"].to(bByVal);
			m_pqxxResultTypes[i]["typisdefined"].to(bIsDefined);
			m_pqxxResultTypes[i]["typelem"].to(oidTypElem);
			m_pqxxResultTypes[i]["typinputoid"].to(oidInputFunc);
			m_pqxxResultTypes[i]["typoutputoid"].to(oidOutputFunc);
			m_pqxxResultTypes[i]["tableoid"].to(oidTable);
				
			pgType->setProperties(
				m_pqxxResultTypes[i]["description"].c_str(), 
				m_pqxxResultTypes[i]["type"].c_str(),
				iTypLen,
				bByVal,
				bIsDefined,
				m_pqxxResultTypes[i]["typdelim"].c_str(),
				m_pqxxResultTypes[i]["align"].c_str(),
				m_pqxxResultTypes[i]["typstorage"].c_str(),
				oidTypElem,
				m_pqxxResultTypes[i]["element"].c_str(),
				oidInputFunc,
				m_pqxxResultTypes[i]["typinput"].c_str(),
				oidOutputFunc,
				m_pqxxResultTypes[i]["typoutput"].c_str(),
				oidTable,
				m_pqxxResultTypes[i]["tablename"].c_str(),
				m_pqxxResultTypes[i]["owner"].c_str()
				);
			
			pgType->refresh(pqxxXaction);
    }
  }
  catch (const std::exception &e)
  {
    kdError() << e.what() << endl;
    KPGSqlDialog dlg(0, strQuery, e.what());
		dlg.exec();
    return false;
  } 
  
  return true;
}

void KPGTypesFolder::displayProperties(QWidget *widgetProperies)
{
	KPGTypesFolderPropPage *pPropertyPage = static_cast <KPGTypesFolderPropPage *> (widgetProperies);
	
	//----------------------------------------------------------------------------
	//Display types list
  QTable *pTableTypes = pPropertyPage->m_pTableTypes;
	
	// fill properties table with result data
	int nTableCols = fillPropertiesTable(m_pqxxResultTypes, pTableTypes);
	
  // field names
  QHeader* hHeader = pTableTypes->horizontalHeader();

  int nCol = 0;
	hHeader->setLabel(nCol++, i18n("Name"));
  hHeader->setLabel(nCol++, i18n("Description"));
	hHeader->setLabel(nCol++, i18n("Type"));
  hHeader->setLabel(nCol++, i18n("Type length"));
	hHeader->setLabel(nCol++, i18n("By value"));
  hHeader->setLabel(nCol++, i18n("Is defined"));
  hHeader->setLabel(nCol++, i18n("Array delimiter"));
  hHeader->setLabel(nCol++, i18n("Align"));
  hHeader->setLabel(nCol++, i18n("Storage"));
	hHeader->setLabel(nCol++, i18n("Element"));
	hHeader->setLabel(nCol++, i18n("Input function"));
	hHeader->setLabel(nCol++, i18n("Output function"));
	hHeader->setLabel(nCol++, i18n("Corresponding object"));
	hHeader->setLabel(nCol++, i18n("Owner"));
	
	for(int nCol = 0; nCol < nTableCols; nCol++)
  {
    pTableTypes->adjustColumn(nCol);
  }
}


