//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcreatefunctionwizard.h"

// include files for Qt
#include <qlayout.h>

// include files for KDE
#include <kdebug.h>
#include <klineedit.h>
#include <ktextedit.h>
#include <kcombobox.h>
#include <kmessagebox.h>
#include <klocale.h>

// application specific includes
#include "../DbObjects/kpgconnection.h"
#include "../DbObjects/kpgserver.h"
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgusersfolder.h"
#include "../DbObjects/kpggroupsfolder.h"
#include "kpgcreatefunctionwidget1.h"
#include "kpgcreatefunctionwidget2.h"
#include "kpgcreatefunctionwidget3.h"
#include "kpgcreatefunctionwidget4.h"

KPGCreateFunctionWizard::KPGCreateFunctionWizard(QWidget *parent, const char *name, KPGDatabase *pDatabase)
 : KPG6StepsWizard(parent, name)
{
	m_pDatabase = pDatabase;
	KPGServer *pServer = static_cast <KPGServer *> (pDatabase->parent());
	
  KPGCreateFunctionWidget1 *pCreateFunctionWidget1 = new KPGCreateFunctionWidget1(WizardPage1, "KPGCreateFunctionWidget1", pDatabase);
  m_pPageWidget1 = pCreateFunctionWidget1;
  WizardPage1Layout->addWidget(m_pPageWidget1, 0, 0);
  connect(pCreateFunctionWidget1, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton1(bool)));
  
  KPGCreateFunctionWidget2 *pCreateFunctionWidget2 = new KPGCreateFunctionWidget2(WizardPage2, "KPGCreateFunctionWidget2");
  m_pPageWidget2 = pCreateFunctionWidget2;
  WizardPage2Layout->addWidget(m_pPageWidget2, 0, 0);
  connect(pCreateFunctionWidget2, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton2(bool)));
  
  KPGCreateFunctionWidget3 *pCreateFunctionWidget3 = new KPGCreateFunctionWidget3(WizardPage3, "KPGCreateFunctionWidget3");
  m_pPageWidget3 = pCreateFunctionWidget3;
  WizardPage3Layout->addWidget(m_pPageWidget3, 0, 0);
  connect(pCreateFunctionWidget3, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton3(bool)));
  
	KPGCreateFunctionWidget4 *pCreateFunctionWidget4 = new KPGCreateFunctionWidget4(WizardPage4, "KPGCreateFunctionWidget4");
  m_pPageWidget4 = pCreateFunctionWidget4;
  WizardPage4Layout->addWidget(m_pPageWidget4, 0, 0);
  connect(pCreateFunctionWidget4, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton4(bool)));
  
	pServer->getGroupsFolder()->fillListBoxWithChildItems(pCreateFunctionWidget4->m_pListBoxUsersGroups);
  pServer->getUsersFolder()->fillListBoxWithChildItems(pCreateFunctionWidget4->m_pListBoxUsersGroups);
   
  languageChange();
  
  setNextEnabled(WizardPage2, true);
}


KPGCreateFunctionWizard::~KPGCreateFunctionWizard()
{
}

// Call it after fill Namespaces combo
void KPGCreateFunctionWizard::initialize()
{
  KPGCreateFunctionWidget1 *pCreateFunctionWidget1 = static_cast <KPGCreateFunctionWidget1 *> (m_pPageWidget1);
  pCreateFunctionWidget1->initialize();
}

void KPGCreateFunctionWizard::languageChange()
{
    KPG6StepsWizard::languageChange();
    
    setCaption( i18n( "Create new function" ) );
    
    setTitle( WizardPage1, i18n( "Step 1 - Function name, return and arguments types" ) );
    setTitle( WizardPage2, i18n( "Step 2 - Function properties" ) );
    setTitle( WizardPage3, i18n( "Step 3 - Function definition" ) );
		setTitle( WizardPage4, i18n( "Step 4 - Grant execute privileges" ) );
    setTitle( WizardPage5, i18n( "Step 5 - Review SQL statement" ) );
    setTitle( WizardPage6, i18n( "Step 6 - Check operation result" ) );
}

// When this function called, initialize appropriate Wizard page
void KPGCreateFunctionWizard::showPage(QWidget *pPage)
{ 

  if(pPage == WizardPage3) // Step 3 - Define function definition
  {
    //--- Initialize 2nd page
    KPGCreateFunctionWidget1 *pCreateFunctionWidget1 = static_cast <KPGCreateFunctionWidget1 *> (m_pPageWidget1);
    KPGCreateFunctionWidget3 *pCreateFunctionWidget3 = static_cast <KPGCreateFunctionWidget3 *> (m_pPageWidget3);
    
    // set appropriate page
    if(pCreateFunctionWidget1->m_pComboBoxLanguage->currentText() == "C")
      pCreateFunctionWidget3->switchToPage(0);
    else
      pCreateFunctionWidget3->switchToPage(1);
  }

	if(pPage == WizardPage4) // Step 4 - Define user rights
	{
		KPGCreateFunctionWidget1 *pCreateFunctionWidget1 = static_cast <KPGCreateFunctionWidget1 *> (m_pPageWidget1);
		KPGCreateFunctionWidget4 *pCreateFunctionWidget4 = static_cast <KPGCreateFunctionWidget4 *> (m_pPageWidget4);
				
		// Set function name and schema
		pCreateFunctionWidget4->setFunctionName(pCreateFunctionWidget1->getFuncNameParams());
		pCreateFunctionWidget4->setNamespace(pCreateFunctionWidget1->m_pComboBoxSchema->currentText());
	}
	
  if(pPage == WizardPage5) // Step 5 - Review SQL statement
  {
    // prepare SQL statement
    QString strSQL;
    strSQL.append((static_cast <KPGCreateFunctionWidget1 *> (m_pPageWidget1))->getSQL());
		strSQL.append((static_cast <KPGCreateFunctionWidget2 *> (m_pPageWidget2))->getSQL());
		strSQL.append((static_cast <KPGCreateFunctionWidget3 *> (m_pPageWidget3))->getSQL());
		strSQL.append(";\n");
		strSQL.append((static_cast <KPGCreateFunctionWidget4 *> (m_pPageWidget4))->getSQL());
    
    
    m_pTextEditSQL->setText(strSQL);
  }
  
  if(pPage == WizardPage6) // Step 6 - Check operation result
  {
    // Run SQL statement
    if(m_pDatabase == 0)
    {
      kdError() << "KPGCreateFunctionWizard::showPage - Database not connected to server !" << endl;
      m_pTextEditResult->setText("Database not connected to server !");
    }
    else
    { 
      try
      {   
        pqxx::result R = m_pDatabase->runQuery(m_pTextEditSQL->text());
        m_pTextEditResult->setText(i18n("Operation successfull"));
        setFinishEnabled(WizardPage6, true);
      }
      catch (const std::exception &e)
      {
        m_pTextEditResult->setText(e.what());
      } 
    }
  }
  
  // call parent
  KPG6StepsWizard::showPage(pPage);
}


#include "kpgcreatefunctionwizard.moc"
