//
// C++ Implementation: kpgcreatetablewidget4
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcreatetablewidget4.h"

// include files for Qt
#include <qlabel.h>
#include <qcheckbox.h>
#include <qheader.h>
#include <qlistview.h>
#include <qpushbutton.h>

// include files for KDE
#include <kdebug.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <klineedit.h>
#include <kcombobox.h> 
#include <ktextedit.h> 
#include <klistbox.h>

// application specific includes
#include "../DbObjects/kpgtreeitem.h"
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgschema.h"
#include "../DbObjects/kpgfunctionsfolder.h"
#include "../DbObjects/kpgfunction.h"

KPGCreateTableWidget4::KPGCreateTableWidget4(QWidget *parent, const char *name, KPGDatabase *pDatabase)
 : KPGCreateTableWidget4Base(parent, name)
{
	m_pDatabase = pDatabase;
	m_bWhereSaved = false;
	
	m_pListViewIndexes->setSortColumn(-1);
}


KPGCreateTableWidget4::~KPGCreateTableWidget4()
{
}

void KPGCreateTableWidget4::setListOfAvailableColumns(KPGListTableColumnWizInfo &listOfAvailableColumns)
{
	m_listOfAvailableColumns = listOfAvailableColumns;
	
	m_pListBoxAvailableCols->clear();
	m_pListBoxIndexCols->clear();
	
	for(unsigned int i= 0; i < m_listOfAvailableColumns.count(); i++)
		{
			m_pListBoxAvailableCols->insertItem(
				* static_cast <KPGTableColumnWizInfo *> (m_listOfAvailableColumns.at(i))->icon(), 
				static_cast <KPGTableColumnWizInfo *> (m_listOfAvailableColumns.at(i))->name()
				);
		}
}

void KPGCreateTableWidget4::slotCurrentFunctionChanged(int iIndex)
{
	if(iIndex > 0)
	{
		if(!m_bWhereSaved)
			{
			  m_strSavedWherePredicate = m_pTextEditWherePredicate->text(); 
				m_bWhereSaved = true;
			}
			
		m_pTextEditWherePredicate->setText("");
		m_pTextEditWherePredicate->setEnabled(false);	
	}
	else
	{
		m_pTextEditWherePredicate->setText(m_strSavedWherePredicate);
		m_pTextEditWherePredicate->setEnabled(true);
		m_bWhereSaved = false;	
	}
}

/////////////////////////////////////////////////////////////////////
//                 INDEX COLUMNS FUNCTIONS
/////////////////////////////////////////////////////////////////////	

void KPGCreateTableWidget4::slotColumnAdd()
{
 	QListBoxItem *pSelectedItem = m_pListBoxAvailableCols->selectedItem();
	if(!pSelectedItem) return;
	
	// Test, if types aren't multidimensional - then warning user
	KPGTableColumnWizInfo *pColumnInfoAvail = m_listOfAvailableColumns.findColumn(m_pListBoxAvailableCols->currentText());
	
	if(pColumnInfoAvail->nDimensions() > 0)
	{
		QString strAvailColumn(i18n("column: ") + pColumnInfoAvail->name() + " " + pColumnInfoAvail->typName() + "\n");
		
		if(KMessageBox::questionYesNo(this, i18n("Multidimensional column - add it anyway ?\n") + strAvailColumn) == KMessageBox::No) return;
	}
	//-------------------------------------------------------------------
	
	m_pListBoxIndexCols->insertItem(* pSelectedItem->pixmap(), pSelectedItem->text());
	m_pListBoxAvailableCols->removeItem(m_pListBoxAvailableCols->currentItem());
	
	m_pPushButtonColumnAdd->setEnabled(false);
	
	setDefaultIndexName();
			
	enableAddIndexButton( m_pLineEditIndexName->text().isEmpty());
	
	refreshListOfFunctions(m_pListBoxIndexCols->count());
}

void KPGCreateTableWidget4::slotColumnDelete()
{
  QListBoxItem *pSelectedItem = m_pListBoxIndexCols->selectedItem();
	if(!pSelectedItem) return;
	
	m_pListBoxAvailableCols->insertItem(* pSelectedItem->pixmap(), pSelectedItem->text());
	m_pListBoxIndexCols->removeItem(m_pListBoxIndexCols->currentItem());
	
	m_pPushButtonColumnDel->setEnabled(false);
	m_pPushButtonColumnUp->setEnabled(false);
	m_pPushButtonColumnDown->setEnabled(false); 
	enableAddIndexButton(m_pLineEditIndexName->text().isEmpty());
	
	refreshListOfFunctions(m_pListBoxIndexCols->count());
}

void KPGCreateTableWidget4::slotColumnUp()
{
  QListBoxItem *pSelectedItem = m_pListBoxIndexCols->selectedItem();
	if(!pSelectedItem) return;
		
	int iIndex = m_pListBoxIndexCols->index(pSelectedItem) - 1;
	if(iIndex < 0) 
		{
			m_pPushButtonColumnUp->setEnabled(false);
			return;
		}
		
	m_pListBoxIndexCols->takeItem(pSelectedItem);
	m_pListBoxIndexCols->insertItem(pSelectedItem, iIndex);
	m_pListBoxIndexCols->setCurrentItem(pSelectedItem);
	
	m_pPushButtonColumnDown->setEnabled(true);
	
	if(iIndex == 0) 
		m_pPushButtonColumnUp->setEnabled(false);
}

void KPGCreateTableWidget4::slotColumnDown()
{
  QListBoxItem *pSelectedItem = m_pListBoxIndexCols->selectedItem();
	if(!pSelectedItem) return;
		
	int iIndex = m_pListBoxIndexCols->index(pSelectedItem) + 1;
	if(iIndex >= (int) m_pListBoxIndexCols->count()) 
		{
			m_pPushButtonColumnDown->setEnabled(false);
			return;
		}
		
	m_pListBoxIndexCols->takeItem(pSelectedItem);
	m_pListBoxIndexCols->insertItem(pSelectedItem, iIndex);
	m_pListBoxIndexCols->setCurrentItem(pSelectedItem);
	
	m_pPushButtonColumnUp->setEnabled(true);
	
	if(iIndex == (int) m_pListBoxIndexCols->count() - 1) 
		m_pPushButtonColumnDown->setEnabled(false); 
}

void KPGCreateTableWidget4::slotAvailableColsSelectChanged(QListBoxItem *pSelectedItem)
{
	m_pPushButtonColumnAdd->setEnabled(pSelectedItem != 0);
}

void KPGCreateTableWidget4::slotIndexColsSelectChanged(QListBoxItem *pSelectedItem)
{
	m_pPushButtonColumnDel->setEnabled(pSelectedItem != 0);
	m_pPushButtonColumnUp->setEnabled(pSelectedItem != 0);
	m_pPushButtonColumnDown->setEnabled(pSelectedItem != 0);
}

void KPGCreateTableWidget4::refreshListOfFunctions(int nArguments)
{
	m_pComboBoxFunctions->clear();
	if(nArguments == 0) return;

	m_pComboBoxFunctions->insertItem(""); // void item for index without function
	
	//--- Obtain list of "immutable" functions with required number of parameters
	
  // First find schema - this same as created table
  KPGTreeItem *pItem = m_pDatabase->getChildByName(m_strNamespace);
  if(!pItem)
  {
    kdDebug() << "KPGCreateTableWidget4::refreshListOfFunctions: no item found: " << m_strNamespace << endl;
    return;
  }
        
  KPGSchema *pSchema = static_cast <KPGSchema *> (pItem);
  KPGFunctionsFolder *pFunctionsFolder = pSchema->getFunctionsFolder();
  
	// traverse list of functions
  QListViewItem * pLvItem = pFunctionsFolder->firstChild();
  while(pLvItem)
  {
    KPGFunction *pFunction = static_cast <KPGFunction *> (pLvItem);
      
    if((pFunction->getVolatile() == "i") && (pFunction->nArgs() == nArguments))
      m_pComboBoxFunctions->insertItem(* pFunction->pixmap(0), pFunction->text(0)); 
      
    pLvItem = pLvItem->nextSibling();
  }
	
	//------------------------------------------------------------------
	
	// Second, find schema - pg_catalog
	pItem = m_pDatabase->getChildByName("pg_catalog");
  if(!pItem)
  {
    kdDebug() << "KPGCreateTableWidget4::refreshListOfFunctions: no item found: " << "pg_catalog" << endl;
    return;
  }
        
  pSchema = static_cast <KPGSchema *> (pItem);
  pFunctionsFolder = pSchema->getFunctionsFolder();
	
	// traverse list of functions
  pLvItem = pFunctionsFolder->firstChild();
  while(pLvItem)
  {
    KPGFunction *pFunction = static_cast <KPGFunction *> (pLvItem);
      
    if((pFunction->getVolatile() == "i") && (pFunction->nArgs() == nArguments))
      m_pComboBoxFunctions->insertItem(* pFunction->pixmap(0), pFunction->text(0)); 
      
    pLvItem = pLvItem->nextSibling();
  }
}

/////////////////////////////////////////////////////////////////////
//                     COMMON FUNCTIONS
/////////////////////////////////////////////////////////////////////	


void KPGCreateTableWidget4::setDefaultIndexName()
{
	QString strFKName("idx_" + m_strTableName + "__");
			
	for(unsigned int i = 0; i < m_pListBoxIndexCols->count(); i++)
			strFKName.append("_" + m_pListBoxIndexCols->text(i));
								
	m_pLineEditIndexName->setText(strFKName);
							
}

void KPGCreateTableWidget4::slotIndexNameChanged(const QString& strText)
{
	enableAddIndexButton(strText.isEmpty());
}

void KPGCreateTableWidget4::enableAddIndexButton(bool bIndexNameEmpty)
{
	if(bIndexNameEmpty)
		m_pPushButtonIndexAdd->setEnabled(false);
	else
		m_pPushButtonIndexAdd->setEnabled(m_pListBoxIndexCols->count() > 0);
}

void KPGCreateTableWidget4::slotIndexAdd()
{
	QListViewItem *pLastItem = 0;
	
	QListViewItem * pItem = m_pListViewIndexes->firstChild();
	while(pItem)
	{
		pLastItem = pItem;
		pItem = pItem->nextSibling();
	}
	
	QString strSQL("CREATE");
	
	if(m_pCheckBoxUnique->isChecked())
		strSQL.append(" UNIQUE");
	
	strSQL.append(" INDEX \"");
		
	strSQL.append(m_pLineEditIndexName->text());
	strSQL.append("\" ON ");
	
	if(!m_strNamespace.isEmpty())
		{
		  strSQL.append(m_strNamespace);
			strSQL.append(".");
	  }
		
	strSQL.append(m_strTableName);
	
	if(m_pComboBoxMethod->currentItem() > 0)
		strSQL.append(" USING " + m_pComboBoxMethod->currentText()); // USING ...
	
	strSQL.append(" (");
		
	if(m_pComboBoxFunctions->currentItem() > 0)
		strSQL.append(" " + m_pComboBoxFunctions->currentText() + "("); // (func_name(column1, column2, ... ))
		
	// List of index columns -----------------------------------------	
	for(unsigned int i = 0; i < m_pListBoxIndexCols->count(); i++)
	{
		strSQL.append("\"");
		strSQL.append(m_pListBoxIndexCols->text(i));
		strSQL.append("\"");
		
		if(i < m_pListBoxIndexCols->count() - 1)
			strSQL.append(", ");
	}
	strSQL.append(")");
	//----------------------------------------------------------------
	
	if(m_pComboBoxFunctions->currentItem() > 0)
		strSQL.append(")");
	
  if(m_pComboBoxTablespace->currentItem() > 0)
    strSQL.append(" TABLESPACE " + m_pComboBoxTablespace->currentText());  
    
	if(m_pTextEditWherePredicate->text().length() > 0)
	{
		strSQL.append(" WHERE ");
		strSQL.append("(" + m_pTextEditWherePredicate->text() + ")");
	}
		
	// Create item for new index
	QListViewItem *pNewItem;
	if(pLastItem == 0) 
		pNewItem = new QListViewItem(m_pListViewIndexes, strSQL);
	else
		pNewItem = new QListViewItem(m_pListViewIndexes, pLastItem, strSQL);
	
	pNewItem->setPixmap(0, * KPGTreeItem::m_pIconIndex);
	
	// Renew available columns list, clear index columns list
	m_pListBoxIndexCols->clear();
	m_pListBoxAvailableCols->clear();
	
	for(unsigned int i= 0; i < m_listOfAvailableColumns.count(); i++)
		{
			m_pListBoxAvailableCols->insertItem(
				* static_cast <KPGTableColumnWizInfo *> (m_listOfAvailableColumns.at(i))->icon(), 
				static_cast <KPGTableColumnWizInfo *> (m_listOfAvailableColumns.at(i))->name()
				);
		}
		
	// Clean up dialog	
	m_pLineEditIndexName->setText("");
	m_pComboBoxFunctions->clear();
	m_pTextEditWherePredicate->setText("");
	m_pTextEditWherePredicate->setEnabled(true);
	sigEnableNextButton(true);
}

void KPGCreateTableWidget4::slotIndexDelete()
{
	QListViewItem *pSelectedItem = m_pListViewIndexes->selectedItem();
	if(pSelectedItem)
			delete pSelectedItem;
	
	m_pPushButtonIndexDel->setEnabled(false);
	
	enableAddIndexButton(m_pLineEditIndexName->text().isEmpty());
	sigEnableNextButton(m_pListViewIndexes->childCount() > 0);
}

void KPGCreateTableWidget4::slotIndexesSelectionChanged(QListViewItem* pItem)
{
	m_pPushButtonIndexDel->setEnabled(pItem != 0);
}

void KPGCreateTableWidget4::slotIndexesCurrentChanged(QListViewItem* pItem)
{
	m_pPushButtonIndexDel->setEnabled(pItem != 0);
}

// Return part of SQL statement for CREATE TABLE
const QString KPGCreateTableWidget4::getSQL() const
{
	QString strSQL;
	
	QListViewItem * pItem = m_pListViewIndexes->firstChild();
	while(pItem)
	{
		strSQL.append("\n");
		strSQL.append(pItem->text(0));
		strSQL.append(";");
					  
		pItem = pItem->nextSibling();
	}
	
	return strSQL;
}


#include "kpgcreatetablewidget4.moc"
