//
// C++ Implementation: kpgcreateuserwidget2
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgcreateuserwidget2.h"

// include files for Qt
#include <qpushbutton.h>

// include files for KDE
#include <klistbox.h>
#include <kmessagebox.h>
#include <kdebug.h>

// application specific includes
#include "../DbObjects/kpgtreeitem.h"
#include "../DbObjects/kpgconnection.h"
#include "../kpgsqldialog.h"

KPGCreateUserWidget2::KPGCreateUserWidget2(QWidget *parent, const char *name)
 : KPGCreateUserWidget2Base(parent, name)
{
}


KPGCreateUserWidget2::~KPGCreateUserWidget2()
{
}

// Set connection for wizard operation
void KPGCreateUserWidget2::setConnection(KPGConnection * pDbConnection)
{
	m_pDbConnection = pDbConnection;
}


void KPGCreateUserWidget2::refreshAvailGroupsList()
{
	// obtain list of groups
	QString strQuery("SELECT groname AS name FROM pg_group ");
	strQuery.append("ORDER BY name ");
		
	try
	{
		m_pListBoxAvailGroups->clear();
			
		pqxx::work pqxxXaction(*m_pDbConnection, "KPGCreateUserWidget2::refreshAvailGropupsList");
		pqxx::result pqxxResultTables = pqxxXaction.Exec(strQuery.latin1());
			
		for (result::size_type i = 0; i != pqxxResultTables.size(); ++i)
		{
			m_pListBoxAvailGroups->insertItem(* KPGTreeItem::m_pIconGroup, pqxxResultTables[i]["name"].c_str());
		}
	}
	catch (const std::exception &e)
	{
		kdError() << e.what() << endl;
		KPGSqlDialog dlg(this, strQuery, e.what());
		dlg.exec();
		return;
	}
}

void KPGCreateUserWidget2::slotAvailableGroupsSelectChanged(QListBoxItem* pSelectedItem)
{
	m_pPushButtonGroupAdd->setEnabled(pSelectedItem != 0);
}
		
void KPGCreateUserWidget2::slotUsersGroupsSelectChanged(QListBoxItem* pSelectedItem)
{
	m_pPushButtonGroupDelete->setEnabled(pSelectedItem != 0);
}

void KPGCreateUserWidget2::slotGroupAdd()
{
	QListBoxItem *pSelectedItem = m_pListBoxAvailGroups->selectedItem();
	if(!pSelectedItem) return;
	
	// Move group from available list to user list 
	m_pListBoxUsersGroups->insertItem(* pSelectedItem->pixmap(), pSelectedItem->text());
		
	m_pListBoxAvailGroups->removeItem(m_pListBoxAvailGroups->currentItem());
	m_pPushButtonGroupAdd->setEnabled(false);
}

void KPGCreateUserWidget2::slotGroupDelete()
{
	QListBoxItem *pSelectedItem = m_pListBoxUsersGroups->selectedItem();
	if(!pSelectedItem) return;
	
	m_pListBoxAvailGroups->insertItem(* pSelectedItem->pixmap(), pSelectedItem->text());
	m_pListBoxUsersGroups->removeItem(m_pListBoxUsersGroups->currentItem());
	
	m_pPushButtonGroupDelete->setEnabled(false);
}		
		
// Return part of SQL statement for CREATE USER
const QString KPGCreateUserWidget2::getSQL() const
{
	QString strSQL("");

	for(unsigned int i = 0; i < m_pListBoxUsersGroups->count(); i++)
	{
		if(i > 0)
			strSQL.append(", ");
		else
			strSQL.append(" IN GROUP ");
			
		strSQL.append(m_pListBoxUsersGroups->item(i)->text());
	}
	
	return strSQL;
}

#include "kpgcreateuserwidget2.moc"
