//
// C++ Implementation: %{MODULE}
//
// Description:
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgmodifyfunctionwizard.h"

// include files for Qt
#include <qlayout.h>
#include <qradiobutton.h>

// include files for KDE
#include <kdebug.h>
#include <klineedit.h>
#include <ktextedit.h>
#include <kcombobox.h>
#include <kmessagebox.h>
#include <klocale.h>

// application specific includes
#include "../DbObjects/kpgconnection.h"
#include "../DbObjects/kpgserver.h"
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgfunction.h"
#include "kpgcreatefunctionwidget2.h"
#include "kpgcreatefunctionwidget3.h"


KPGModifyFunctionWizard::KPGModifyFunctionWizard(QWidget *parent, const char *name, KPGFunction *pFunction)
 : KPG4StepsWizard(parent, name)
{
  // use 2nd and 3rd pages from create function Wizard
  
  KPGCreateFunctionWidget2 *pCreateFunctionWidget2 = new KPGCreateFunctionWidget2(WizardPage1, "KPGCreateFunctionWidget2");
  m_pPageWidget1 = pCreateFunctionWidget2;
  WizardPage1Layout->addWidget(m_pPageWidget1, 0, 0);
  connect(pCreateFunctionWidget2, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton1(bool)));
  
  KPGCreateFunctionWidget3 *pCreateFunctionWidget3 = new KPGCreateFunctionWidget3(WizardPage2, "KPGCreateFunctionWidget3");
  m_pPageWidget2 = pCreateFunctionWidget3;
  WizardPage2Layout->addWidget(m_pPageWidget2, 0, 0);
  connect(pCreateFunctionWidget3, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton2(bool)));
  
  languageChange();
	
	m_strFunctionName = pFunction->text(0);
	m_strFunctionName.append("(" + pFunction->argTypes() + ")");
	m_strReturnTypName = pFunction->returnTypName();
	m_strLanName = pFunction->langName();
	
	// setip 1st page
	if(pFunction->getVolatile() == "s")
		pCreateFunctionWidget2->m_pRadioButtonStable->setChecked(true);
		 
	if(pFunction->getVolatile() == "v")
		pCreateFunctionWidget2->m_pRadioButtonVolatile->setChecked(true);
		
	// NULL arguments beghacvior
	if(pFunction->isStrict())
		pCreateFunctionWidget2->m_pRadioButtonStrict->setChecked(true);
	
	// Security
	if(pFunction->isSecdef())
		pCreateFunctionWidget2->m_pRadioButtonSecurityDefiner->setChecked(true);	
		
	// Set function definition
	pCreateFunctionWidget3->m_pTextEditFunctionDefinition->setText(pFunction->source());
	pCreateFunctionWidget3->switchToPage(1);
		
	setNextEnabled(WizardPage1, true);	
	setNextEnabled(WizardPage2, true);	
}

KPGModifyFunctionWizard::~KPGModifyFunctionWizard()
{
}

void KPGModifyFunctionWizard::languageChange()
{
    KPG4StepsWizard::languageChange();
    
    setCaption( i18n( "Modify function" ) );
    
    setTitle( WizardPage1, i18n( "Step 1 - Function properties" ) );
    setTitle( WizardPage2, i18n( "Step 2 - Function definition" ) );
    setTitle( WizardPage3, i18n( "Step 3 - Review SQL statement" ) );
    setTitle( WizardPage4, i18n( "Step 4 - Check operation result" ) );
}

// When this function called, initialize appropriate Wizard page
void KPGModifyFunctionWizard::showPage(QWidget *pPage)
{ 
  if(pPage == WizardPage3) // Step 3 - Review SQL statement
	{
		// prepare SQL statement
		QString strSQL("CREATE OR REPLACE FUNCTION ");
		
		strSQL.append(m_strFunctionName);
		strSQL.append("\nRETURNS ");
		strSQL.append(m_strReturnTypName);
		strSQL.append("\nLANGUAGE ");
		strSQL.append(m_strLanName);
		strSQL.append("\n");
		
		strSQL.append((static_cast <KPGCreateFunctionWidget2 *> (m_pPageWidget1))->getSQL());
		strSQL.append((static_cast <KPGCreateFunctionWidget3 *> (m_pPageWidget2))->getSQL());
		strSQL.append(";");
				
		m_pTextEditSQL->setText(strSQL);
	}
	
	if(pPage == WizardPage4) // Step 4 - Check operation result
	{
		// Run SQL statement
		if(m_pDatabase == 0)
		{
			kdError() << "KPGModifyFunctionWizard::showPage - Database not connected to server !" << endl;
			m_pTextEditResult->setText("Database not connected to server !");
		}
		else
		{	
			try
  		{		
				pqxx::result R = m_pDatabase->runQuery(m_pTextEditSQL->text());
				m_pTextEditResult->setText(i18n("Operation successfull"));
				setFinishEnabled(WizardPage4, true);
			}
			catch (const std::exception &e)
			{
				m_pTextEditResult->setText(e.what());
			} 
		}
	}
	
	// call parent
  KPG4StepsWizard::showPage(pPage);
}


#include "kpgmodifyfunctionwizard.moc"
