//
// C++ Implementation: kpgmodifyviewwizard
//
// Description: 
//
//
// Author: Lumir Vanek <lvanek@users.sourceforge.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kpgmodifyviewwizard.h"

// include files for Qt
#include <qlayout.h>

// include files for KDE
#include <kdebug.h>
#include <ktextedit.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <ktextedit.h>

// application specific includes
#include "../DbObjects/kpgdatabase.h"
#include "../DbObjects/kpgview.h"
#include "kpgmodifyviewwidget1.h"


KPGModifyViewWizard::KPGModifyViewWizard(QWidget *parent, const char *name, KPGView *pView)
 : KPG3StepsWizard(parent, name)
{
	KPGModifyViewWidget1 *pModifyViewWidget1 = new KPGModifyViewWidget1(WizardPage1, "KPGModifyViewWidget1");
	m_pPageWidget1 = pModifyViewWidget1;
	WizardPage1Layout->addWidget(m_pPageWidget1, 0, 0);
	connect(pModifyViewWidget1, SIGNAL(sigEnableNextButton(bool)), this, SLOT(slotEnableNextButton1(bool)));

	languageChange();
	
	m_strViewName = pView->text(0);
	// set View definition to edit box
	pModifyViewWidget1->m_pTextEditSQL->setText(pView->definition());
}


KPGModifyViewWizard::~KPGModifyViewWizard()
{
}

void KPGModifyViewWizard::languageChange()
{
    KPG3StepsWizard::languageChange();
		
		setCaption( i18n( "Create new View" ) );
		
    setTitle( WizardPage1, i18n( "Step 1 - Enter SQL statement for view" ) );
}

// When this function called, initialize appropriate Wizard page
void KPGModifyViewWizard::showPage(QWidget *pPage)
{
	if(pPage == WizardPage1) // Step 1 - Define view tables
	{
		//--- Initialize 1st page
		KPGModifyViewWidget1 *pModifyViewWidget1 = static_cast <KPGModifyViewWidget1 *> (m_pPageWidget1);

		// set database
		pModifyViewWidget1->setDatabase(m_pDatabase);
	}

	if(pPage == WizardPage2) // Step 2 - Review SQL statement
	{
		// prepare SQL statement
		QString strSQL("CREATE OR REPLACE VIEW ");
		
		strSQL.append(m_strViewName);
		strSQL.append(" AS\n");
		
		strSQL.append((static_cast <KPGModifyViewWidget1 *> (m_pPageWidget1))->getSQL());
		
		m_pTextEditSQL->setText(strSQL);
	}
	
	if(pPage == WizardPage3) // Step 3 - Check operation result
	{
		// Run SQL statement
		if(m_pDatabase == 0)
		{
			kdError() << "KPGModifyViewWizard::showPage - Database not connected to server !" << endl;
			m_pTextEditResult->setText("Database not connected to server !");
		}
		else
		{	
			try
  		{		
				pqxx::result R = m_pDatabase->runQuery(m_pTextEditSQL->text());
				m_pTextEditResult->setText(i18n("Operation successfull"));
				setFinishEnabled(WizardPage3, true);
			}
			catch (const std::exception &e)
			{
				m_pTextEditResult->setText(e.what());
			} 
		}
	}
	
	// call parent
	KPG3StepsWizard::showPage(pPage);
	
}
#include "kpgmodifyviewwizard.moc"
