/***************************************************************************
                          kpgsyntaxhighlighter.cpp  -  description
                             -------------------
    begin                : � led 13 2004
    copyright            : (C) 2004 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpgsyntaxhighlighter.h"

#include <iostream>

#include <qcolor.h>
#include <qregexp.h>
#include <qstring.h>
#include <qstringlist.h>

#include <ktextedit.h>

// SQL language keywords list
#define NUM_KEYWORDS 120
const char *arStrKeyWords[NUM_KEYWORDS] = {
"ABORT",
"AGGREGATE",
"ALL",
"ALTER",
"ARRAY",
"AS",
"ASC",
"BEGIN",
"BY",
"CASE",
"CAST",
"CLOSE",
"CLUSTER",
"COMMENT",
"COMMIT",
"CONSTRAINT",
"CONSTRAINTS",
"COPY",
"CREATE",
"CROSS",
"CURRENT_DATE",
"CURRENT_TIME",
"CURRENT_TIMESTAMP",
"CURRENT_USER",
"DATABASE",
"DECLARE",
"DEFAULT",
"DEFERRABLE",
"DEFERRED",
"DELETE",
"DESC",
"DISTINCT",
"DROP",
"ELSE",
"EXIT",
"END",
"EXCEPT",
"EXPLAIN",
"FALSE",
"FETCH",
"FOR",
"FOREIGN",
"FOUND",
"FROM",
"FULL",
"FUNCTION",
"GRANT",
"GROUP",
"HAVING",
"CHECK",
"IMMEDIATE",
"INDEX",
"INHERITS",
"INITIALLY",
"INNER",
"INSERT",
"INTERSECT",
"INTO",
"IF",
"IS",
"JOIN",
"KEY",
"LANGUAGE",
"LEFT",
"LIMIT",
"LISTEN",
"LOAD",
"LOCK",
"LOOP",
"MATCH",
"MOVE",
"NATURAL",
"NOTIFY",
"NULL",
"OFFSET",
"ON",
"ONLY",
"OPEN",
"OPERATOR",
"ORDER",
"OUTER",
"PARTIAL",
"PRIMARY",
"REFERENCES",
"REINDEX",
"RENAME",
"REPLACE",
"RESET",
"RESTRICT",
"RESULT",
"RETURN",
"RETURNS",
"REVOKE",
"ROLLBACK",
"RULE",
"SELECT",
"SEQUENCE",
"SET",
"SET",
"SHOW",
"TABLE",
"THEN",
"TRANSACTION",
"TRIGGER",
"TRUE",
"TRUNCATE",
"TYPE",
"UNION",
"UNIQUE",
"UNLISTEN",
"UPDATE",
"USER",
"USING",
"VACUUM",
"VALUES",
"VIEW",
"VOLATILE",
"WHEN",
"WHILE",
"WHERE" };

// SQL language datatypes list
#define NUM_DATATYPES 43
const char *arStrDataTypes[NUM_DATATYPES] = {
"boolean",
"bool",
"bit",
"varying",
"character",
"char",
"varchar",
"text",
"smallint",
"int2",
"integer",
"int",
"int4",
"bigint",
"int8",
"real",
"float4",
"double",
"precision",
"float8",
"float",
"numeric",
"decimal",
"money",
"serial",
"date",
"time",
"with",
"zone",
"timestamp",
"interval",
"box",
"line",
"lseg",
"circle",
"path",
"point",
"polygon",
"cidr",
"inet",
"macadr",
"oid",
"xid" };

// SQL language operators list
#define NUM_OPERATORS 35
const char *arStrOperators[NUM_OPERATORS] = {
"+",
"-",
"/",
"*",
"%",
"^",
"|/",
"||/",
"!",
"!!",
"@",
"<",
">",
"<=",
">=",
"~",
"!~",
"~*",
"!~*",
"AND",
"OR",
"NOT",
"=",
"!=",
"<>",
"&",
"|",
"#",
"<<",
">>",
"BETWEEN",
"IN",
"LIKE",
"ILIKE",
"OVERLAPS" };

/*
Functions
nextval
curval

*/

KPGSyntaxHighlighter::KPGSyntaxHighlighter(QTextEdit *textEdit)
  : QSyntaxHighlighter(textEdit)
{
  m_clrDefaultText.setRgb(0, 0, 0);
  m_clrKeyWord.setRgb(128, 0, 0);
  m_clrDataType.setRgb(0, 255, 255);
  m_clrOperator.setRgb(0, 255, 0);
  m_clrQuotedString.setRgb(0, 0, 128);
  m_clrNumber.setRgb(255, 255, 0);
  m_clrComment.setRgb(128, 128, 128);
}

KPGSyntaxHighlighter::~KPGSyntaxHighlighter()
{
}


int KPGSyntaxHighlighter::highlightParagraph(const QString& text, int endStateOfLastPara)
{
    //first I format the given line to default so any remaining highlighting is removed (Qt does not do it by itself)
    setFormat(0 , text.length(), QColor(0, 0, 0));

    int pos;
		unsigned int i = 0;
	
		if(endStateOfLastPara == 1)
		{
			for(i = 0; i < text.length(); i++)
    	{
				QRegExp patternComment("[^*]*[^/]*\\*/"); // search end of comment
        pos=patternComment.search(text, i);

        if(pos >= 0) // end comment found ?
        {
          int l = patternComment.matchedLength();
              
          setFormat(0, l, m_clrComment);
          i += l; // skip comment
					break;
        }
				else
				{
					setFormat(0, text.length(), m_clrComment);
					return 1; // return 1 to signify "in comment"
				}
			}
		}
		
		
    for(; i < text.length(); i++)
    {
      if((text[i] == '-') && (i+1 < text.length()) && (text[i+1] == '-'))
      {
        // simple comment found
        setFormat(i, text.length(), m_clrComment );
        break; // skip rest of line
      }
			
			if((text[i] == '/') && (i+1 < text.length()) && (text[i+1] == '*'))
			{
				QRegExp patternComment("/\\*[^*]*[^/]*\\*/"); // search comment
        pos=patternComment.search(text, i);

        if((unsigned int) pos == i) // closed comment found ?
        {
          int l = patternComment.matchedLength();
              
          setFormat(pos, l, m_clrComment);
          i += l; // skip comment
        }
				else
				{
					setFormat(i, text.length() - i, m_clrComment);
					return 1; // return 1 to signify "in comment"
				}
			}
      
      if(text[i] != ' ')
      {
        // try find keyword
        for(int j = 0; j < NUM_KEYWORDS; j++)
        {
          if(QChar(text[i]).lower() != QChar(arStrKeyWords[j][0]).lower())
            continue; // test first character for speed up  

          QString strPattern("\\b");
	  			strPattern.append(arStrKeyWords[j]);
          strPattern.append("\\b");
          QRegExp patternKeyWord(strPattern, false); // search keyword
          pos=patternKeyWord.search(text, i);
          if((unsigned int) pos == i)
          {
            int l = patternKeyWord.matchedLength();
            setFormat(i, l, m_clrKeyWord );
            i += l;
            break;
          } 
        }

        // try find datatype
        for(int j = 0; j < NUM_DATATYPES; j++)
        {
          if(QChar(text[i]).lower() != QChar(arStrDataTypes[j][0]).lower())
            continue; // test first character for speed up

	  			QString strPattern("\\b");  
          strPattern.append(arStrDataTypes[j]);
          strPattern.append("\\b");
          QRegExp patternDataType(strPattern, false); // search datatype
          pos=patternDataType.search(text, i);
          if((unsigned int) pos == i)
          {
            int l = patternDataType.matchedLength();
            setFormat(i, l, m_clrDataType );
            i += l;
            break;
          }
        }

        // try find operators
        for(int j = 0; j < NUM_OPERATORS; j++)
        {
          if(QChar(text[i]).lower() != QChar(arStrOperators[j][0]).lower())
            continue; // test first character for speed up

					QString strPattern("\\b");  
          strPattern.append(arStrOperators[j]);
          strPattern.append("\\b");
          QRegExp patternOperators(strPattern, false); // search datatype
          pos=patternOperators.search(text, i);
          if((unsigned int) pos == i)
          {
            int l = patternOperators.matchedLength();
            if(l > 0)
            {
              setFormat(i, l, m_clrOperator );
              i += l;
            }
            break;
          }
        }

        // try find quoted string
        if((text[i] == '\'') || (text[i] == '\"'))
        {
          QRegExp patternQuotedString("\"[^<\"]*\"|'[^<']*'"); // search quoted string
          pos=patternQuotedString.search(text, i);

          if((unsigned int) pos == i) 
            {
              int l = patternQuotedString.matchedLength();

              setFormat(i+1, l - 2, m_clrQuotedString);
              
              i += l;
            }
        }

        // try find numbers
       if((text[i] >= '0') && (text[i] <= '9'))
        {
          
					QString strPattern("\\b\\d+\\b");
	  		
					QRegExp patternNumber(strPattern, false); // search numbers
        	pos=patternNumber.search(text, i);
        	if((unsigned int) pos == i)
        	{
          	int l = patternNumber.matchedLength();
          	setFormat(i, l, m_clrNumber );
          	i += l;
        	}
				} 
      }
    }

    return 0;
}

