/***************************************************************************
                          kpogre.cpp  -  description
                             -------------------
    begin                : � �n  6 16:08:39 CEST 2002
    copyright            : (C) 2002 by Dominik Sacher
    email                : dsacher@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qinputdialog.h>
#include <stdlib.h>
#include <qcombobox.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
#include <kcursor.h>
#include <kdebug.h>

// application specific includes
#include "kpogre.h"
#include "kpogredoc.h"
#include "kpgconfiguration.h"
#include "kpgbookmarksbutton.h"
#include "DbObjects/kpgserver.h"
#include "DbObjects/kpgdatabase.h"
#include "DbObjects/kpgschema.h"
#include "DbObjects/kpgfunction.h"
#include "DbObjects/kpgtablespace.h"
#include "DbObjects/kpgtablecolumn.h"
#include "kpgsearchdialog.h"
#include "kpogrecomboaction.h"

#define ID_STATUS_MSG 1

KPGConfiguration * KPoGreApp::s_pKPGConfig = 0L;

KPoGreApp::KPoGreApp(QWidget* , const char* name)
  : KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();
	
  readOptions();

	m_pDlgSearch = 0;
  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  m_pActFileSave->setEnabled(false);
  m_pActFileSaveAs->setEnabled(false);
  m_pActFilePrint->setEnabled(false);
  m_pActEditCut->setEnabled(false);
  m_pActEditCopy->setEnabled(false);
  m_pActEditPaste->setEnabled(false);
}

KPoGreApp::~KPoGreApp()
{
  if ( s_pKPGConfig )
		delete s_pKPGConfig;

	s_pKPGConfig = 0L;

  if(m_pBookmarksButton)
    delete m_pBookmarksButton;
		
	if(m_pDlgSearch)
		delete m_pDlgSearch; 
}

void KPoGreApp::initActions()
{
  //-----------------------------------------------------------------
	// Database menu       
  m_pActDatabaseConnect = new KAction(i18n("&Connect..."), "connect_established", 0, this, SLOT(slotDatabaseConnect()), actionCollection(), "connect");
	m_pActDatabaseDisconnect = new KAction(i18n("&Disconnect"), "connect_no", 0, this, SLOT(slotDatabaseDisconnect()), actionCollection(), "disconnect");
  m_pActDatabaseRunQuery = new KAction(i18n("&Run query"), "fork", Key_F8, this, SLOT(slotDatabaseRunQuery()), actionCollection(), "run_query");
  m_pActDatabaseWizard = new KAction(i18n("&Wizard..."), "wizard", Key_F9, this, SLOT(slotDatabaseWizard()), actionCollection(), "wizard");
	
	//-----------------------------------------------------------------
	// File menu
  m_pActFileExportResult = new KAction(i18n("&Export Result..."), "fileexport", 0, this, SLOT(slotExportResult()), actionCollection(), "export_result");

	m_pActFileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  m_pActFileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  m_pActFileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  m_pActFileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  m_pActFileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  m_pActFileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  m_pActFileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  m_pActFilePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  m_pActFileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());

  m_pActFileNewWindow->setStatusText(i18n("Opens a new application window"));
  m_pActFileNew->setStatusText(i18n("Creates a new document"));
  m_pActFileOpen->setStatusText(i18n("Opens an existing document"));
  m_pActFileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  m_pActFileSave->setStatusText(i18n("Saves the actual document"));
  m_pActFileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  m_pActFileClose->setStatusText(i18n("Closes the actual document"));
  m_pActFilePrint ->setStatusText(i18n("Prints out the actual document"));
  m_pActFileQuit->setStatusText(i18n("Quits the application"));

  //-----------------------------------------------------------------
	// Edit menu
	m_pActEditUndo = KStdAction::undo(this, SLOT(slotEditUndo()), actionCollection(),"undo");
  m_pActEditRedo = KStdAction::redo(this, SLOT(slotEditRedo()), actionCollection(),"redo");
								
	m_pActEditCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  m_pActEditCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  m_pActEditPaste = new KPasteTextAction(i18n("&Paste"), "editpaste", CTRL+Key_P, this, SLOT(slotEditPaste()), actionCollection(), "edit_paste");
	
	m_pActEditFind = KStdAction::find( this, SLOT(slotEditFind()), actionCollection(), "find" );
	m_pActEditFindNext = KStdAction::findNext( this, SLOT(slotEditFindNext()), actionCollection(), "findNext" );
															
  /*m_pActEditCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  m_pActEditCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  m_pActEditPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));*/

	 m_pActEditClearInputArea = new KAction(i18n("Clear &Inputarea"), "editdelete", 0, this, SLOT(slotEditClearInputArea()), actionCollection(), "clear_input_area");

	
	m_pActEditDeselect = new KAction( i18n("D&eselect Node"), CTRL+Key_E, this,
	                                SLOT(slotEditDeselect()), actionCollection(), "deselect" );

  //-----------------------------------------------------------------
	// View menu
  m_pActViewNodeUp = new KAction( i18n("To &Parent Item"), "up", 0, this,
	                                SLOT(slotViewNodeUp()), actionCollection(), "treeitem_up" );
																	
	m_pActViewNodeBack = new KAction( i18n("To &Back Item"), "back", 0, this,
	                                SLOT(slotViewNodeBack()), actionCollection(), "treeitem_back" );
																	
	m_pActViewNodeForward = new KAction( i18n("To &Forward Item"), "forward", 0, this,
	                                SLOT(slotViewNodeForward()), actionCollection(), "treeitem_forward" );
																	
	m_pActViewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection(), "showtoolbar");
  m_pActViewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection(), "showstatusbar");
 
  //-----------------------------------------------------------------
	// Settings menu
	KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection(), "configurekeys");
  KStdAction::configureToolbars(this, SLOT(slotConfigToolbars()), actionCollection(), "configuretoolbars");
	KStdAction::preferences(this, SLOT(slotConfigure()), actionCollection(), "configure");
	
  //-----------------------------------------------------------------
	//-----------------------------------------------------------------
	// Popup menus
  //-----------------------------------------------------------------
  //-----------------------------------------------------------------
  
	m_pActFolderRefresh = new KAction( i18n("&Refresh"), "reload", Key_F5, this, SLOT(slotRefresh()), actionCollection(), "refresh" );
	
	//--- Database related menu actions
	
	(void) new KAction( i18n("&Create database..."), "wizard", 0, this, SLOT(slotCreateDatabase()), actionCollection(), "create_database" );
	
	(void) new KAction( i18n("&Drop database..."), "editdelete", 0, this, SLOT(slotDropDatabase()), actionCollection(), "drop_database" );
 
  (void) new KAction( i18n("Disconnect from database"), "connect_no", 0, this, SLOT(slotDisconnectDatabase()), actionCollection(), "disconnect_database" );
   
	
	//--- Schema related menu actions
	
	(void) new KAction( i18n("&Create schema..."), "wizard", 0, this, SLOT(slotCreateSchema()), actionCollection(), "create_schema" );
	
	(void) new KAction( i18n("&Drop schema..."), "editdelete", 0, this, SLOT(slotDropSchema()), actionCollection(), "drop_schema" );
	
	(void) new KAction( i18n("&Rename schema..."), "edittool", 0, this, SLOT(slotModifySchema()), actionCollection(), "rename_schema" );
	
	//--- Table related menu actions
	
	(void) new KAction( i18n("&Create table..."), "wizard", 0, this, SLOT(slotCreateTable()), actionCollection(), "create_table" );
	
	(void) new KAction( i18n("&Drop table..."), "editdelete", 0, this, SLOT(slotDropTable()), actionCollection(), "drop_table" );
	
  (void) new KAction( i18n("&Rename table..."), "edittool", 0, this, SLOT(slotRenameTable()), actionCollection(), "rename_table" );
  
	//--- View related menu actions
	
	(void) new KAction( i18n("&Create view..."), "wizard", 0, this, SLOT(slotCreateView()), actionCollection(), "create_view" );
	
	(void) new KAction( i18n("&Drop view..."), "editdelete", 0, this, SLOT(slotDropView()), actionCollection(), "drop_view" );
	
	(void) new KAction( i18n("&Modify view..."), "edit", 0, this, SLOT(slotModifyView()), actionCollection(), "modify_view" );

  //--- Tablespaces related menu actions
  
  (void) new KAction( i18n("&Create tablespace..."), "wizard", 0, this, SLOT(slotCreateTablespace()), actionCollection(), "create_tablespace" );
  
  (void) new KAction( i18n("&Drop tablespace..."), "editdelete", 0, this, SLOT(slotDropTablespace()), actionCollection(), "drop_tablespace" );
  
  (void) new KAction( i18n("&Modify tablespace..."), "edit", 0, this, SLOT(slotModifyTablespace()), actionCollection(), "modify_tablespace" );
  
  
	//--- User related menu actions
	
	(void) new KAction( i18n("&Create user..."), "wizard", 0, this, SLOT(slotCreateUser()), actionCollection(), "create_user" );
	
	(void) new KAction( i18n("&Drop user..."), "editdelete", 0, this, SLOT(slotDropUser()), actionCollection(), "drop_user" );
	
	(void) new KAction( i18n("&Modify user..."), "edit", 0, this, SLOT(slotModifyUser()), actionCollection(), "modify_user" );
	
	//--- Group related menu actions
	
	(void) new KAction( i18n("&Create group..."), "wizard", 0, this, SLOT(slotCreateGroup()), actionCollection(), "create_group" );
	
	(void) new KAction( i18n("&Drop group..."), "editdelete", 0, this, SLOT(slotDropGroup()), actionCollection(), "drop_group" );
	
	(void) new KAction( i18n("&Modify group..."), "edit", 0, this, SLOT(slotModifyGroup()), actionCollection(), "modify_group" );
  
  //--- Language related menu actions
  
  (void) new KAction( i18n("&Create language..."), "wizard", 0, this, SLOT(slotCreateLanguage()), actionCollection(), "create_language" );
  
  (void) new KAction( i18n("&Drop language..."), "editdelete", 0, this, SLOT(slotDropLanguage()), actionCollection(), "drop_language" );
  
  (void) new KAction( i18n("&Rename language..."), "edittool", 0, this, SLOT(slotModifyLanguage()), actionCollection(), "rename_language" );
    
  //--- Function related menu actions
  
  (void) new KAction( i18n("&Create function..."), "wizard", 0, this, SLOT(slotCreateFunction()), actionCollection(), "create_function" );
  
  (void) new KAction( i18n("&Drop function..."), "editdelete", 0, this, SLOT(slotDropFunction()), actionCollection(), "drop_function" );
  
  (void) new KAction( i18n("&Rename function..."), "edittool", 0, this, SLOT(slotRenameFunction()), actionCollection(), "rename_function" );

  (void) new KAction( i18n("&Modify function..."), "edit", 0, this, SLOT(slotModifyFunction()), actionCollection(), "modify_function" );
      
  //--- Sequence related menu actions
  
  (void) new KAction( i18n("&Create sequence..."), "wizard", 0, this, SLOT(slotCreateSequence()), actionCollection(), "create_sequence" );
  
  (void) new KAction( i18n("&Drop sequence..."), "editdelete", 0, this, SLOT(slotDropSequence()), actionCollection(), "drop_sequence" );
  
  (void) new KAction( i18n("&Modify sequence..."), "edit", 0, this, SLOT(slotModifySequence()), actionCollection(), "modify_sequence" );
  
  //--- Column related menu actions
  
  (void) new KAction( i18n("&Create column(s)..."), "wizard", 0, this, SLOT(slotCreateColumn()), actionCollection(), "create_columns" );
  
  (void) new KAction( i18n("&Drop column..."), "editdelete", 0, this, SLOT(slotDropColumn()), actionCollection(), "drop_column" );
  
  (void) new KAction( i18n("&Rename column..."), "edittool", 0, this, SLOT(slotRenameColumn()), actionCollection(), "rename_column" );

  (void) new KAction( i18n("Set &NOT NULL..."), 0, 0, this, SLOT(slotSetColumnNull()), actionCollection(), "set_column_null" );
  (void) new KAction( i18n("Set d&efault..."), 0, 0, this, SLOT(slotSetColumnDefault()), actionCollection(), "set_column_default" );
  (void) new KAction( i18n("Dr&op default..."), 0, 0, this, SLOT(slotDropColumnDefault()), actionCollection(), "drop_column_default" );
  (void) new KAction( i18n("Change &type..."), 0, 0, this, SLOT(slotChangeColumnType()), actionCollection(), "change_column_type" );
  (void) new KAction( i18n("Set &storage..."), 0, 0, this, SLOT(slotSetColumnStorage()), actionCollection(), "set_column_storage" );
  (void) new KAction( i18n("Set st&atistics..."), 0, 0, this, SLOT(slotSetColumnStatistisc()), actionCollection(), "set_column_stat" );
      
  
  //--- Constraint related menu actions
  
  (void) new KAction( i18n("&Create constraint(s)..."), "wizard", 0, this, SLOT(slotCreateConstraint()), actionCollection(), "create_constraints" );
  
  (void) new KAction( i18n("&Drop constraint..."), "editdelete", 0, this, SLOT(slotDropConstraint()), actionCollection(), "drop_constraint" );
  
	//--- Index related menu actions
  
  (void) new KAction( i18n("&Create index(es)..."), "wizard", 0, this, SLOT(slotCreateIndex()), actionCollection(), "create_indexes" );
  
  (void) new KAction( i18n("&Drop index..."), "editdelete", 0, this, SLOT(slotDropIndex()), actionCollection(), "drop_index" );
  
  (void) new KAction( i18n("&Rename index..."), "edittool", 0, this, SLOT(slotRenameIndex()), actionCollection(), "rename_index" );
  
  //--- Rule related menu actions
  
  (void) new KAction( i18n("&Create rule..."), "wizard", 0, this, SLOT(slotCreateRule()), actionCollection(), "create_rule" );
  
  (void) new KAction( i18n("&Drop rule..."), "editdelete", 0, this, SLOT(slotDropRule()), actionCollection(), "drop_rule" );
  
  //--- Trigger related menu actions
  
  (void) new KAction( i18n("&Create trigger..."), "wizard", 0, this, SLOT(slotCreateTrigger()), actionCollection(), "create_trigger" );
  
  (void) new KAction( i18n("&Drop trigger..."), "editdelete", 0, this, SLOT(slotDropTrigger()), actionCollection(), "drop_trigger" );
  
  (void) new KAction( i18n("&Rename trigger..."), "edittool", 0, this, SLOT(slotRenameTrigger()), actionCollection(), "rename_trigger" );
  

  //-----------------------------------------------------------------
  //-----------------------------------------------------------------
	// path toolbar
  //-----------------------------------------------------------------
  //-----------------------------------------------------------------
	m_pActPathCombo = new KPoGreComboAction( i18n("Path Bar"), 0, this,                                    SLOT(slotPathSelected(const QString &)), actionCollection(), "path_combo" );
	
	new KAction( i18n("Clear Path Bar"), BarIcon("locationbar_erase", 16), 0, this, SLOT(slotPathClear()), actionCollection(), "path_clear" );
	
	QLabel *locationLabel = new ToolbarLabel( i18n("Path: ") );
	
	new KWidgetAction( locationLabel, i18n("Path: "), 0, this, 0, actionCollection(), "path_label" );
	
	locationLabel->setBuddy( m_pActPathCombo->comboBox());
			
  //-----------------------------------------------------------------
	// use the absolute path to your kpogreui.rc file for testing purpose in createGUI();
  //createGUI("/home/lvanek/Projects/Own/kpogre/kpogre/kpogreui.rc"); 
  createGUI();

  // ... create the bookmark list
  KToolBar * pdbToolBar = toolBar("dbToolBar");

  m_pBookmarksButton = new KPGBookmarksButton( pdbToolBar );
  connect( m_pBookmarksButton, SIGNAL( pressed() ), this, SLOT( slotBookmarksButtonPressed() ) );
  m_pBookmarksButton->enableAddBookmarkMenu(false);
  connect( m_pBookmarksButton, SIGNAL( openUrl( const QString& ) ), this, SLOT( slotOpenUrl( const QString& ) ) );
	
  // initial state
  m_pActDatabaseDisconnect->setEnabled(false);
	m_pActDatabaseWizard->setEnabled(false);
	
	m_pActFileExportResult->setEnabled(false);
	m_pActFileNew->setEnabled(true);
  m_pActFileOpen->setEnabled(true);
  m_pActFileOpenRecent->setEnabled(true);
  m_pActFileSave->setEnabled(false);
  m_pActFileSaveAs->setEnabled(false);
  m_pActFileClose->setEnabled(false);
	
	m_pActEditFindNext->setEnabled(true);
	m_pActEditDeselect->setEnabled(false);
	
	m_pActViewNodeUp->setEnabled(false);
	m_pActViewNodeBack->setEnabled(false);
	m_pActViewNodeForward->setEnabled(false);
	
	m_pActDatabaseRunQuery->setEnabled(false);
}


void KPoGreApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void KPoGreApp::initDocument()
{
  doc = new KPoGreDoc(this);
  doc->newDocument();
	connect(doc, SIGNAL(sigModified()), this, SLOT(slotDocumentModified()));
}

void KPoGreApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KPoGreView(this);
  doc->addView(view);
  setCentralWidget(view);
  setCaption(doc->URL().fileName(),false);

  connect( view, SIGNAL(sigSelectionChanged(const KPGTreeItem::eNodeType, KPGTreeItem *, unsigned int, unsigned int)), this, SLOT(slotSelectionChanged(const KPGTreeItem::eNodeType, KPGTreeItem *, unsigned int, unsigned int)) );
	connect( view, SIGNAL(sigEnableRunQueryAction(bool)), this, SLOT(slotEnableRunQueryAction(bool)) );
	connect( view, SIGNAL(sigEnableFileActions(bool, bool, bool)), this, SLOT(slotEnableFileActions(bool, bool, bool)) );
	connect( view, SIGNAL(sigEnableEditActions(bool, bool, bool, bool, bool, bool)), this, SLOT(slotEnableEditActions(bool, bool, bool, bool, bool, bool)) );
	connect( view, SIGNAL(sigContextMenuRequested(const QString&,const QPoint&)), this, SLOT(slotContextMenuRequested(const QString&,const QPoint&)) );
}

void KPoGreApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  m_pActFileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
}


KPoGreDoc *KPoGreApp::getDocument() const
{
  return doc;
}

void KPoGreApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", m_pActViewToolBar->isChecked());
  config->writeEntry("Show Statusbar",m_pActViewStatusBar->isChecked());
  config->writeEntry("mainToolBarPos", (int) toolBar("mainToolBar")->barPos());
	config->writeEntry("pathToolBarPos", (int) toolBar("pathToolBar")->barPos());
	config->writeEntry("dbToolBarPos", (int) toolBar("dbToolBar")->barPos());
  m_pActFileOpenRecent->saveEntries(config,"Recent Files");
}

void KPoGreApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  m_pActViewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  m_pActViewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();

  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("mainToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);
	
	toolBarPos=(KToolBar::BarPosition) config->readNumEntry("pathToolBarPos", KToolBar::Top);
  toolBar("pathToolBar")->setBarPos(toolBarPos);
	
	toolBarPos=(KToolBar::BarPosition) config->readNumEntry("dbToolBarPos", KToolBar::Left);
  toolBar("dbToolBar")->setBarPos(toolBarPos);
	
  // initialize the recent file list
  m_pActFileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

KPGConfiguration * KPoGreApp::configuration()
{
	if ( ! s_pKPGConfig )
		s_pKPGConfig = new KPGConfiguration();

	return s_pKPGConfig;
}

// TODO: remove it
/*
void KPoGreApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();	
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}*/

// TODO: remove it
/*void KPoGreApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);
  	
    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}*/		

bool KPoGreApp::queryClose()
{
  return doc->saveModified();
}

bool KPoGreApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
//                     ACTION SLOTS
/////////////////////////////////////////////////////////////////////

void KPoGreApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));
	
  KPoGreApp *new_window= new KPoGreApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    doc->newDocument();		
    setCaption(doc->URL().fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {	
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*.sql|SQL files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(), false);
      m_pActFileOpenRecent->addURL( url );
    }
  }
  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	
  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    doc->openDocument(url);
    setCaption(url.fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	
	if(doc->URL().fileName() == i18n("Untitled"))
  	slotFileSaveAs();
	else
		doc->saveDocument(doc->URL());
		
	setCaption(doc->URL().fileName(), doc->isModified());

  m_pActFileSave->setEnabled(false);

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*.sql|SQL files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    doc->saveDocument(url);
    m_pActFileOpenRecent->addURL(url);
    setCaption(url.fileName(), doc->isModified());
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFilePrint()
{
  slotStatusMsg(i18n("Printing..."));

  QPrinter printer;
  if (printer.setup(this))
  {
    view->print(&printer);
  }

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotFileQuit()
{
  slotStatusMsg(i18n("Exiting..."));
  saveOptions();
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
	break;
    }
  }	
}

// Undo last edit action in SQL query edit-widget
void KPoGreApp::slotEditUndo()
{
	slotStatusMsg(i18n("Undo last edit action in SQL query"));
	view->editUndo();
  slotStatusMsg(i18n("Ready."));
}

// Redo last edit action in SQL query edit-widget
void KPoGreApp::slotEditRedo()
{
	slotStatusMsg(i18n("Redo last edit action in SQL query"));
	view->editRedo();
  slotStatusMsg(i18n("Ready."));
}		 

void KPoGreApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));
	view->editCut();
  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));
	view->editCopy();
  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));
	view->editPaste();
  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotEditFind()
{
	slotStatusMsg( i18n("Search in objects tree ...") );

	if ( ! m_pDlgSearch )
	{
		m_pDlgSearch = new KPGSearchDialog( widget(), "search dialog", true );
	}

	if( m_pDlgSearch->exec() == KPGSearchDialog::Accepted )
		slotEditFindNext();
	
	m_pActEditFindNext->setEnabled(true);

	slotStatusMsg( i18n("Ready.") );
}

void KPoGreApp::slotEditFindNext()
{
	slotStatusMsg( i18n("Search in objects tree ...") );

	if ( ! m_pDlgSearch )
	{
		kdDebug() << "KPoGreApp::slotEditFindNext implementation error - no search dialog" << endl;
		slotStatusMsg( i18n("Ready.") );
		return;
	}

	view->editFind(m_pDlgSearch->getSearchString(), m_pDlgSearch->getOIDs(), m_pDlgSearch->getInNames());

	slotStatusMsg( i18n("Ready.") );
}

void KPoGreApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!m_pActViewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }		

  slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!m_pActViewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

/** Connect to Database Server */
void KPoGreApp::slotDatabaseConnect()
{
	slotStatusMsg(i18n("Connect to Database ..."));
	setCursor(KCursor::waitCursor());
	
	view->databaseConnect();
	
	setCursor(KCursor::arrowCursor());
	slotStatusMsg(i18n("Ready."));
}

/** Disconnect Database */
void KPoGreApp::slotDatabaseDisconnect()
{
	slotStatusMsg(i18n("Disconnect from Database"));
	view->databaseDisconnect();
	slotStatusMsg(i18n("Ready."));
}

void KPoGreApp::slotDatabaseRunQuery()
{
  view->databaseRunQuery();
}

void KPoGreApp::slotDatabaseWizard()
{
  view->databaseWizard();
}

/** Clear Input Area */
void KPoGreApp::slotEditClearInputArea()
{
	slotStatusMsg(i18n("Clear Inputarea"));
	view->editClearInputArea();
	slotStatusMsg(i18n("Ready."));
}
/** Dialog for kye binding configuration */
void KPoGreApp::slotConfigureKeys()
{
  KKeyDialog::configure(actionCollection(), xmlFile());
}
/** Configure toolbars dialog */
void KPoGreApp::slotConfigToolbars()
{
  saveMainWindowSettings(KGlobal::config(), "KPogreMainWindow");
  KEditToolbar dlg(actionCollection());

	if (dlg.exec())
		{ createGUI();
		}
}

/** Configure application */
void KPoGreApp::slotConfigure()
{
	slotStatusMsg( i18n("Configure KpoGre ...") );

	configuration()->showDialog();

	slotStatusMsg( i18n("Ready.") );
}

/** Save query result to file */
void KPoGreApp::slotExportResult()
{
	QString fileName = KFileDialog::getSaveFileName( QString(getenv("HOME") ), QString("*.csv"), view, QString("Export query-result as ...") );
	if( fileName != QString::null ) {			// OK, User made or selected a filename
		QFile* file = new QFile( fileName );
  		if( file->exists() )
			{ 
			  //is it already existing ?
  			QString error(fileName); 	
				error.append(i18n(" already exists!\n Do you wish to overwrite it ?"));
  	  	if (KMessageBox::questionYesNo(view, error, i18n("Overwrite File ?"))==KMessageBox::No) 
				{
  	  			slotExportResult();					//user does not want to overwrite ! -> jump out and recall
  	  			return;
  	  	}
  	  }
		
  	  	if ( ! file->open(IO_WriteOnly) ) {  // is he able to write to it ?
  	  		QString error(i18n("Error writing to "));
  	  		error.append(fileName);
  	  		error.append(" !");
  	  		KMessageBox::error( view, error, i18n("Error"));
  	  		return;
  	  	}
 		
  	  	else {   	  	                    //Yes, he is able to write !
  			QStringList list;
  			list << ";" << "|" << " ";
  			QString sep = QInputDialog::getItem( i18n("Field Separator ?"), i18n("Please choose a field separator !"), list  );
  			QTextStream* out = new QTextStream( file );
     		QTable* outputArea = view->getCurrentlySelectedTableResult();
        if(!outputArea)
          return;
          
			  int cols = outputArea->numCols();
     		int rows = outputArea->numRows();
     		for( int currentRow=0 ; currentRow<rows ; currentRow++ ){
     			for (int currentCol=0 ; currentCol<cols ; currentCol++ ){
     				*out << (outputArea->text( currentRow, currentCol )).stripWhiteSpace() << sep;
     			}
				*out << "\n";
			}
			file->flush();
			file->close();
		}
	}
}

// Called when user press Bookmarks Button
void KPoGreApp::slotBookmarksButtonPressed()
{
  m_pBookmarksButton->openPopup();
}


// Called when user choose bookmark
void KPoGreApp::slotOpenUrl( const QString& strUrl)
{
  view->openUrl(strUrl);
}

// Refresh selected item
void KPoGreApp::slotRefresh()
{
	setCursor(KCursor::waitCursor());
	view->refresh();
	setCursor(KCursor::arrowCursor());
}

// Run Create Database Wizard 
void KPoGreApp::slotCreateDatabase()
{
	view->createDatabase();
}

// Run Drop Database Wizard 
void KPoGreApp::slotDropDatabase()
{
	view->dropDatabase();
}

// Close connection to database
void KPoGreApp::slotDisconnectDatabase()
{
  view->disconnectDatabase();
}

// Run Create Schema Wizard 
void KPoGreApp::slotCreateSchema()
{
	view->createSchema();
}

// Run Drop Schema Wizard 
void KPoGreApp::slotDropSchema()
{
	view->dropSchema();
}

// Run Create or Replace Schema Wizard 
void KPoGreApp::slotModifySchema()
{
	view->modifySchema();
}

// Run Create Table Wizard 
void KPoGreApp::slotCreateTable()
{
	view->createTable();
}

// Run Drop Table Wizard 
void KPoGreApp::slotDropTable()
{
	view->dropTable();
}

// Run Alter Table Rename Wizard 
void KPoGreApp::slotRenameTable()
{
  view->renameTable();
}

// Run Create View Wizard 
void KPoGreApp::slotCreateView()
{
	view->createView();
}

// Run Drop View Wizard 
void KPoGreApp::slotDropView()
{
	view->dropView();
}

// Run Create or Replace View Wizard 
void KPoGreApp::slotModifyView()
{
	view->modifyView();
}

// Run Create Tablespace Wizard 
void KPoGreApp::slotCreateTablespace()
{
  view->createTablespace();
}

// Run Drpp Tablespace Wizard 
void KPoGreApp::slotDropTablespace()
{
  view->dropTablespace();
}

// Run Alter Tablespace Wizard 
void KPoGreApp::slotModifyTablespace()
{
  view->modifyTablespace();
}


// Run Create User Wizard 
void KPoGreApp::slotCreateUser()
{
	view->createUser();
}

// Run Drpp User Wizard 
void KPoGreApp::slotDropUser()
{
	view->dropUser();
}

// Run Alter User Wizard 
void KPoGreApp::slotModifyUser()
{
	view->modifyUser();
}

// Run Create View Group 
void KPoGreApp::slotCreateGroup()
{
	view->createGroup();
}

// Run Drpp Group Wizard 
void KPoGreApp::slotDropGroup()
{
	view->dropGroup();
}

// Run Alter Group Wizard 
void KPoGreApp::slotModifyGroup()
{
	view->modifyGroup();
}

// Run Create Language Wizard 
void KPoGreApp::slotCreateLanguage()
{
  view->createLanguage();
}
  
// Run Drop Language Wizard 
void KPoGreApp::slotDropLanguage()
{
  view->dropLanguage();
}
    
// Run Alter Language Wizard 
void KPoGreApp::slotModifyLanguage()
{
  view->modifyLanguage();
}
    
// Run Create Function Wizard 
void KPoGreApp::slotCreateFunction()
{
  view->createFunction();
}
  
// Run Drop Function Wizard 
void KPoGreApp::slotDropFunction()
{
  view->dropFunction();
}
   
// Run Alter Function Wizard 
void KPoGreApp::slotRenameFunction()
{
  view->renameFunction();
}
 
// Run Alter Function Wizard 
void KPoGreApp::slotModifyFunction()
{
  view->modifyFunction();
}

// Run Create Sequence Wizard 
void KPoGreApp::slotCreateSequence()
{
  view->createSequence();
}
  
// Run Drop Sequence Wizard 
void KPoGreApp::slotDropSequence()
{
  view->dropSequence();
}

// Run ALTER Sequence Wizard 
void KPoGreApp::slotModifySequence()
{
  view->modifySequence();
}

// Run ALTER table ADD Column Wizard
void KPoGreApp::slotCreateColumn()
{
  view->createColumn();
}
    
// Run ALTER table DROP Column Wizard
void KPoGreApp::slotDropColumn()
{
  view->dropColumn();
}
    
// Run ALTER table Rename Column Wizard
void KPoGreApp::slotRenameColumn()
{
  view->renameColumn();
}
    
// Run ALTER table SET/DROP Column NULL Wizard
void KPoGreApp::slotSetColumnNull()
{
  view->setColumnNull();
}
    
// Run ALTER table ALTER Column SET DEFAULT Wizard
void KPoGreApp::slotSetColumnDefault()
{
  view->setColumnDefault();
}

// Run ALTER table ALTER Column DROP DEFAULT Wizard
void KPoGreApp::slotDropColumnDefault()
{
  view->dropColumnDefault();
}
    
// Run ALTER table ALTER Column TYPE Wizard
void KPoGreApp::slotChangeColumnType()
{
  view->changeColumnType();
}
    
// Run ALTER table ALTER Column SET STORAGE { PLAIN | EXTERNAL | EXTENDED | MAIN } Wizard
void KPoGreApp::slotSetColumnStorage()
{
  view->setColumnStorage();
}
    
// Run ALTER table ALTER Column SET STATISTICS Wizard
void KPoGreApp::slotSetColumnStatistisc()
{
  view->setColumnStatistisc();
}    

    
// Run Create Constraint Wizard 
void KPoGreApp::slotCreateConstraint()
{
  view->createConstraint();
}
  
// Run Drop Constraint Wizard 
void KPoGreApp::slotDropConstraint()
{
  view->dropConstraint();
}

// Run Create Index Wizard 
void KPoGreApp::slotCreateIndex()
{
  view->createIndex();
}
  
// Run Drop Index Wizard 
void KPoGreApp::slotDropIndex()
{
  view->dropIndex();
}

// Run ALTER Index RENAME Wizard 
void KPoGreApp::slotRenameIndex()
{
  view->renameIndex();
}

// Run Create Rule Wizard 
void KPoGreApp::slotCreateRule()
{
  view->createRule();
}
  
// Run Drop Rule Wizard 
void KPoGreApp::slotDropRule()
{
  view->dropRule();
}

// Run Create Trigger Wizard 
void KPoGreApp::slotCreateTrigger()
{
  view->createTrigger();
}
  
// Run Drop Trigger Wizard 
void KPoGreApp::slotDropTrigger()
{
  view->dropTrigger();
}

// Run ALTER Trigger RENAME Wizard 
void KPoGreApp::slotRenameTrigger()
{
  view->renameTrigger();
}

void KPoGreApp::slotEditDeselect() 
{ 
	view->editDeselect(); 
}

void KPoGreApp::slotPathSelected( const QString & strPath )
{
	if(!view->selectItemByPath(strPath))
	{ // node not found, remove item from combo
		m_pActPathCombo->removeItem(strPath);
		m_pActPathCombo->slotClearEdit();
	}
}

void KPoGreApp::slotPathClear()
{
	slotEditDeselect();
	m_pActPathCombo->slotFocusEdit();
}


/////////////////////////////////////////////////////////////////////
//                           OTHER SLOTS
/////////////////////////////////////////////////////////////////////

void KPoGreApp::slotSelectionChanged( const KPGTreeItem::eNodeType nodeType, KPGTreeItem * pItem, unsigned int iStackBackCount, unsigned int iStackForwardCount)
{
	bool bActDatabaseWizard = false;
	bool bActDatabaseDisconnect = false;
	bool bAddBookmarkMenu = false;
	
	if(!pItem)
	{
		m_pActViewNodeUp->setEnabled(false);
		m_pActEditDeselect->setEnabled(false);
	}
	else
	{
		m_pActViewNodeUp->setEnabled(true);
		m_pActEditDeselect->setEnabled(false);
	}
	
	m_pActViewNodeBack->setEnabled(iStackBackCount > 0);
	m_pActViewNodeForward->setEnabled(iStackForwardCount > 0);
	
	switch(nodeType)
  {
    case KPGTreeItem::nodeUnselected:
        m_pActDatabaseDisconnect->setEnabled(false);
        break;

    case KPGTreeItem::nodeServer:
      {
        bActDatabaseDisconnect = true;
        bAddBookmarkMenu = true;
				bActDatabaseWizard = true;
				
        KPGServer *pServer = static_cast <KPGServer *> (pItem);            
        m_pBookmarksButton->bookmarkHandler()->setNewBookmarkName(pServer->text(0));
        m_pBookmarksButton->bookmarkHandler()->setNewBookmarkUrl(pServer->url());
        
      }
      break;
				
		case KPGTreeItem::nodeFunction:
      {
        KPGFunction *pFunction = static_cast <KPGFunction *> (pItem);            
        
				KAction *pAct = actionCollection()->action("modify_function");
				if(pAct)
				{
					pAct->setEnabled(pFunction->langName() != "c"); // Don't allow modify C function
				}
      }
      break;
      
    case KPGTreeItem::nodeTablespace:
      {
        KPGTablespace *pTablespace = static_cast <KPGTablespace *> (pItem);            
        
        KAction *pActModify = actionCollection()->action("modify_tablespace");
        if(pActModify)
        {
          pActModify->setEnabled(!pTablespace->isSystemObject()); // Don't allow modify system tablespaces
        }
        
        KAction *pActDrop = actionCollection()->action("drop_tablespace");
        if(pActDrop)
        {
          pActDrop->setEnabled(!pTablespace->isSystemObject()); // Don't allow drop system tablespaces
        }
      }
      break;
		      
    case KPGTreeItem::nodeDatabase:
      {
        bActDatabaseWizard = true;
        
        KPGDatabase *pDatabase = static_cast <KPGDatabase *> (pItem);            
        
        KAction *pActModify = actionCollection()->action("modify_database");
        if(pActModify)
        {
          pActModify->setEnabled(!pDatabase->isSystemObject() && !pDatabase->isTemplate()); // Don't allow modify system databases
        }
        
        KAction *pActDrop = actionCollection()->action("drop_database");
        if(pActDrop)
        {
          pActDrop->setEnabled(!pDatabase->isSystemObject() && !pDatabase->isTemplate()); // Don't allow drop system databases
        }
      }
      break;  
    
    case KPGTreeItem::nodeSchema:
      {
        bActDatabaseWizard = true;
        
        KPGSchema *pSchema = static_cast <KPGSchema *> (pItem);            
        
        KAction *pActModify = actionCollection()->action("rename_schema");
        if(pActModify)
        {
          pActModify->setEnabled(!pSchema->isSystemObject()); // Don't allow rename system schema
        }
        
        KAction *pActDrop = actionCollection()->action("drop_schema");
        if(pActDrop)
        {
          pActDrop->setEnabled(!pSchema->isSystemObject()); // Don't allow drop system schema
        }
      }
        break;
    
    case KPGTreeItem::nodeTable:
      {
        bActDatabaseWizard = true;
      }
      break;     
            	
      
    case KPGTreeItem::nodeTableColumn:
      {
        KPGTableColumn *pColumn = static_cast <KPGTableColumn *> (pItem);
        KPGServer *pServer = static_cast <KPGServer *> (pColumn->getParentByType(KPGTreeItem::nodeServer));
        
        bool bEnableModifyAct = (pColumn->attNum() > 0); // don't allow modify system columns
        bool bVersion8 = false;
        
        if(pServer) bVersion8 = (pServer->versionMajor() >= 8);
        
        actionCollection()->action("drop_column")->setEnabled(bEnableModifyAct);
        actionCollection()->action("rename_column")->setEnabled(bEnableModifyAct);
        KAction *pActSetNull = actionCollection()->action("set_column_null");
        pActSetNull->setEnabled(bEnableModifyAct);
        actionCollection()->action("set_column_default")->setEnabled(bEnableModifyAct);
        actionCollection()->action("drop_column_default")->setEnabled(bEnableModifyAct && pColumn->hasDefault());
        actionCollection()->action("change_column_type")->setEnabled(bEnableModifyAct && bVersion8);
        actionCollection()->action("set_column_storage")->setEnabled(bEnableModifyAct);
        actionCollection()->action("set_column_stat")->setEnabled(bEnableModifyAct);
          
        if(!pColumn->isNotNull())
          pActSetNull->setText(i18n("Set &NOT NULL..."));
        else
          pActSetNull->setText(i18n("Drop &NOT NULL..."));
      }
      break; 
      
		default:	
				break;
  }
	
	m_pActDatabaseDisconnect->setEnabled(bActDatabaseDisconnect);
	m_pActDatabaseWizard->setEnabled(bActDatabaseWizard);
	m_pBookmarksButton->enableAddBookmarkMenu(bAddBookmarkMenu);
	
	// change path combo text
	if(pItem)
		{ 
			QString strPath(pItem->path());
			
			if((pItem->hasOid()))
			{
				KPGObject *pObject = static_cast <KPGObject *> (pItem);
				
				strPath.append(QString("[%1]").arg(pObject->oid()));
			}
		  m_pActPathCombo->insertItem(* pItem->pixmap(0), strPath);
		}
	else
		m_pActPathCombo->slotClear();
	
}

// Display popup menu
void KPoGreApp::slotContextMenuRequested(const QString & strMenuName, const QPoint & pos)
{
	QWidget * pContainer = guiFactory()->container(strMenuName, this);

	if ( ! pContainer )
	{
		kdError() << "KPoGreApp " << k_funcinfo << " Couldn't get a container widget for the given menu name (" << strMenuName << ")" << endl;
		return;
	}

	if ( ! pContainer->inherits("KPopupMenu") )
	{
		kdError() << "KPoGreApp " << k_funcinfo << " Wrong container widget" << endl;
		return;
	}

	KPopupMenu * pMenu = static_cast <KPopupMenu*> (pContainer);
	pMenu->popup( pos );
}

// Called, when document is modified. Enable Save button. */
void KPoGreApp::slotDocumentModified()
{
	m_pActFileSave->setEnabled(true);
	setCaption(doc->URL().fileName(), true);
}

void KPoGreApp::slotEnableRunQueryAction(bool bEnable)
{
  m_pActDatabaseRunQuery->setEnabled(bEnable);
}

void KPoGreApp::slotEnableFileActions(bool bEnableOpen, bool bEnableSave, bool bEnableExport)
{
	//m_pActFileNew->setEnabled(bEnable);
  
	m_pActFileOpen->setEnabled(bEnableOpen);
  m_pActFileOpenRecent->setEnabled(bEnableOpen);
  
	m_pActFileSave->setEnabled(bEnableSave);
  m_pActFileSaveAs->setEnabled(bEnableSave);
  
	m_pActFileExportResult->setEnabled(bEnableExport);
}

void KPoGreApp::slotEnableEditActions(bool bEnableUndo, bool bEnableRedo, bool bEnableCutCopy, bool bEnablePaste, bool bEnableDeselect, bool bEnableClearInpArea)
{
	m_pActEditUndo->setEnabled(bEnableUndo);
	m_pActEditRedo->setEnabled(bEnableRedo);
	
	m_pActEditCut->setEnabled(bEnableCutCopy);
  m_pActEditCopy->setEnabled(bEnableCutCopy);
  m_pActEditPaste->setEnabled(bEnablePaste);
	
	m_pActEditDeselect->setEnabled(bEnableDeselect);
	m_pActEditClearInputArea->setEnabled(bEnableClearInpArea);
}

void KPoGreApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}
