/***************************************************************************
                          kpogreview.h  -  description
                             -------------------
    begin                : Fre Sep  7 16:06:25 CEST 2001
    copyright            : (C) 2002 by Lumir Vanek
    email                : lvanek@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPOGREVIEW_H
#define KPOGREVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for pqxx
#include "pqxx/all.h"

// include files for Qt
#include <qwidget.h>
#include <qlistview.h>
#include <qtextview.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qtable.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qsplitter.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qvaluelist.h>
#include <qdialog.h>
#include <qlineedit.h>
#include <qplatinumstyle.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <qfont.h>
#include <qmessagebox.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qvaluestack.h>

//includes files for kde
#include <kmessagebox.h>
#include <klocale.h>

#include "DbObjects/kpgtreeitem.h"

class KPoGreDoc;
class KPGSqlQueryResultWindow;
class KPGSchemaPropPage;
class KPGServer;
class KPGDatabase;
class KPGSchema;
class KPGTable;

/**
Storage class for OID and their object path. Used for navigating through OIDs.

@author Lumir Vanek
*/

class KPGOidLink
{
public:
		KPGOidLink() {;} 
    KPGOidLink(pqxx::oid _oid, const QString & strPath)
		{
				m_oid = _oid;
				m_strPath = strPath;
		}
					
	~KPGOidLink() {;}

	// Return PostgreSQL row identifier
	const pqxx::oid oid() const { return m_oid; }
		
	// Return path
	const QString & path() const { return m_strPath; }
	
protected:

	QString m_strPath;

	// PostgreSQL row identifier
	pqxx::oid m_oid;		
};



/** The KPoGreView class provides the view widget for the KPoGreApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KPoGreView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KPoGreApp class to manipulate and display
 * the document structure provided by the KPoGreDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KPoGreView : public QWidget
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    KPoGreView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~KPoGreView();
		
/////////////////////////////////////////////////////////////////////
//                     PROTECTED ATTRIBUTES
/////////////////////////////////////////////////////////////////////
protected:

  // Right tab for properties QTable and 10 SQL query windows
  QTabWidget*  m_pTabPropertiesAndTerminals;
  
  // Display properties for acually selected item in left tree
  QWidget*	m_pWidgetProperties;
	
	// Pointer to array of KPGSqlQueryResultWindow * pointers to query/result windows
  KPGSqlQueryResultWindow** m_ppTerminals;
	
	// Vertical spletter betwwen m_pViewTree and properties window
	QSplitter*	m_pSplitterleft;
	
	// Left tree view that displays DB objects tree	
	QListView*  m_pViewTree;

	// Two stask for storing navigation history through OID links
	QValueStack<KPGOidLink> m_stackBackNodes;
	QValueStack<KPGOidLink> m_stackForwardNodes;	
	
/////////////////////////////////////////////////////////////////////
//                     PUBLIC FUNCTIONS
/////////////////////////////////////////////////////////////////////
public:

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a KPoGreApp instance as a parent
    widget to get to the window document pointer by calling the KPoGreApp::getDocument() method.
    @see KPoGreApp#getDocument
    */
  KPoGreDoc *getDocument() const;
		
  /** contains the implementation for printing functionality */
  void print(QPrinter *pPrinter);
	
  // Set the given sql-string from an sqlfile on inputArea
  void setSQLText(const QString sqlText);

	// Return SQL test from input area
	const QString getSQLText() const;

  // Return tree view
  QListView * getTreeView() const { return m_pViewTree; }

  // Return the currently selected result table (e.g. for saving the content to a file
	QTable* getCurrentlySelectedTableResult();
 
	
/////////////////////////////////////////////////////////////////////
//                     PROTECTED FUNCTIONS
/////////////////////////////////////////////////////////////////////	
protected:

	// Select given tree item
	void selectItem(KPGTreeItem * const);
	
	// Select item defined bu OID and path
	KPGTreeItem* findItem(const KPGOidLink & oidLink);
	
	// Walk through listView items
	KPGTreeItem * nextTreeItem(const KPGTreeItem *);
		
	// Create Properties tab, with regards to selected type in tree
  void createPropertiesTab(const KPGTreeItem::eNodeType);
	
	// Send SQL query to server
	void runQuery(const QString &);
	
	// Display SQL result
  void displayResult(pqxx::result, QTable*, const QString &);
	
	// Find parent database and schemas list for item
	KPGDatabase * lookUpParentDatabaseAndListOfSchemas(KPGTreeItem *, QPtrList<KPGSchema> &);
	
	// Refesh items child folders with given name
	void refreshItemsFolder(KPGTreeItem *, const QString &);
	
public:	
/////////////////////////////////////////////////////////////////////
//                     ACTION FUNCTIONS
/////////////////////////////////////////////////////////////////////
	
	// Called when user choose bookmark
  void openUrl( const QString& );
	
	// Connect to PostgreSQL server
	void databaseConnect();
	
	// Disconnect from selected server
	void databaseDisconnect();
	
	// Send text from selected Inputarea to server
	void databaseRunQuery();
	
	// Run Wizard
	void databaseWizard();
	
	// Called when user launch Find or Find Next menu item 
	void editFind(const QString &, bool, bool);
	
	// Clear inputarea in SQL query widget
	void editClearInputArea();
	
	// Deselects currently selected item in trewe viw
	void editDeselect();
	
	// Undo last edit action in SQL query edit-widget
	void editUndo();
		 
	// Redo last edit action in SQL query edit-widget
	void editRedo();
	
	// Cut text from SQL query widget
	void editCut();
	
	// Copy text from SQL query widget
	void editCopy();
	
	// Paste text to SQL query widget
	void editPaste();
	
	// Navigate to parent item
	void viewNodeUp();
	
	// Navigate back
	void viewNodeBack();
	
	// Navigate forward
	void viewNodeForward();	
		
	// Refresh current item
	void refresh();
	
	// Select item in tree view by given path 
	bool selectItemByPath(const QString &);
	
	// Run Create Database Wizard 
	void createDatabase();
		
	// Run Drop Database Wizard 
	void dropDatabase();
  
  // Close connection to database
  void disconnectDatabase();
	
	// Run Create Schema Wizard 
	void createSchema();
		
	// Run Drop Schema Wizard 
	void dropSchema();
	
	// Run Alter Schema Wizard 
	void modifySchema();
	
	// Run Create Table Wizard 
	void createTable();
		
	// Run Drop Table Wizard 
	void dropTable();

  // Run ALTER Table RENAME Wizard 
  void renameTable();
  
	// Run Create View Wizard 
	void createView();
		
	// Run Drop View Wizard 
	void dropView();
	
	// Run Create or Replace View Wizard 
	void modifyView();
	  
  // Run Create Tablespace Wizard 
  void createTablespace();
  
  // Run Drop Tablespace Wizard 
  void dropTablespace();
    
  // Run Alter Tablespace Wizard 
  void modifyTablespace();
  
  // Run Create User Wizard 
	void createUser();
	
	// Run Drop User Wizard 
	void dropUser();
		
	// Run Alter User Wizard 
	void modifyUser();
				
	// Run Create Group Wizard 
	void createGroup();
	
	// Run Drop Group Wizard 
	void dropGroup();
		
	// Run Alter Group Wizard 
	void modifyGroup();
    
  // Run Create Language Wizard 
  void createLanguage();
  
  // Run Drop Language Wizard 
  void dropLanguage();
    
  // Run Alter Language Wizard 
  void modifyLanguage();
    
  // Run Create Function Wizard 
  void createFunction();
  
  // Run Drop Function Wizard 
  void dropFunction();
    
  // Run Alter Function Wizard 
  void renameFunction();
  
  // Run Alter Function Wizard 
  void modifyFunction();
    
  // Run Create Sequence Wizard 
  void createSequence();
  
  // Run Drop Sequence Wizard 
  void dropSequence();
  
   // Run ALTER Sequence Wizard 
  void modifySequence();
  
  // Run ALTER table ADD Column Wizard
  void createColumn();
    
  // Run ALTER table DROP Column Wizard
  void dropColumn();
    
  // Run ALTER table Rename Column Wizard
  void renameColumn();
    
  // Run ALTER table SET/DROP Column NULL Wizard
  void setColumnNull();
    
  // Run ALTER table ALTER Column SET DEFAULT Wizard
  void setColumnDefault();
  
  // Run ALTER table ALTER Column DROP DEFAULT Wizard
  void dropColumnDefault();
  
  // Run ALTER table ALTER Column TYPE Wizard
  void changeColumnType();
    
  // Run ALTER table ALTER Column SET STORAGE { PLAIN | EXTERNAL | EXTENDED | MAIN } Wizard
  void setColumnStorage();
    
  // Run ALTER table ALTER Column SET STATISTICS Wizard
  void setColumnStatistisc();
  
  // Run Create Constraint Wizard 
  void createConstraint();
  
  // Run Drop Constraint Wizard 
  void dropConstraint();
  
	// Run Create Index Wizard 
  void createIndex();
  
  // Run Drop Index Wizard 
  void dropIndex();
  
  // Run ALTER Index RENAME Wizard 
  void renameIndex();
  
  // Run Create Rule Wizard 
  void createRule();
  
  // Run Drop Rule Wizard 
  void dropRule();
    
  // Run Create Trigger Wizard 
  void createTrigger();
  
  // Run Drop Trigger Wizard 
  void dropTrigger();
  
  // Run ALTER Trigger RENAME Wizard 
  void renameTrigger();  
  	
/////////////////////////////////////////////////////////////////////
//                           OTHER SLOTS
/////////////////////////////////////////////////////////////////////
	
protected slots:		
	
	
	// Show popup for item in tree rightpressed
	void slotTreeViewRightButtonPressed(QListViewItem*, const QPoint&, int);
	
	// Called when user change main tab
	void slotTabWidgetCurrentChanged( QWidget *);
	
	// Called when user change main tab inside schema properties window
  void slotSchemaPropTabChanged(KPGSchemaPropPage *, QWidget *);
  
  // Checks, which type of item was selected
	void slotSelectionChanged();
	
	// Caled when user change selected server in SQL query / result window
	void slotRequestDatabasesList(KPGSqlQueryResultWindow *, const QString&);
	
	// Called from property pages when user double-clicks on KPGLinkLabel
	void slotSearchObject(const pqxx::oid);
	
	// Called when user change SQL query in editor
	void slotQueryTextChanged(KPGSqlQueryResultWindow *);
	
	// Called when availability of copy action is changed in SQL query editor
	void slotCopyAvailable(KPGSqlQueryResultWindow *, bool);
	
	// Called when availability of undo action is changed in SQL query editor
	void slotUndoAvailable(KPGSqlQueryResultWindow *, bool);
	
	// Called when availability of redo action is changed in SQL query editor
	void slotRedoAvailable(KPGSqlQueryResultWindow *, bool);

	
/////////////////////////////////////////////////////////////////////
//                       WIZARDS FUNCTIONS
/////////////////////////////////////////////////////////////////////		
	
  bool createTablespaceWizard(KPGServer *);
  
	bool createDatabaseWizard(KPGServer *);
	
	bool createSchemaWizard(KPGDatabase *);

	bool createGroupWizard(KPGServer *);
	
	bool createUserWizard(KPGServer *);
		
	bool createTableWizard(KPGDatabase *, QPtrList<KPGSchema> &);

	bool createViewWizard(KPGDatabase *, QPtrList<KPGSchema> &);
  
  bool createFunctionWizard(KPGDatabase *, QPtrList<KPGSchema> &);
  
	bool createSequenceWizard(KPGDatabase *, QPtrList<KPGSchema> &);
	
  bool createLanguageWizard(KPGDatabase *);
  
  bool createColumnWizard(KPGDatabase *, KPGSchema *, KPGTable *);
  
  bool createConstraintWizard(KPGDatabase *, KPGSchema *, KPGTable *, int = 0);

	bool createIndexWizard(KPGDatabase *, KPGSchema *, KPGTable *);
  
  bool createRuleWizard(KPGDatabase *, KPGSchema *, KPGTable *);
  
  bool createTriggerWizard(KPGDatabase *, KPGSchema *, KPGTable *);
		
	// Run simple 2-steps for dropping DB object
	bool dropObjectWizard(KPGTreeItem *, KPGTreeItem::eNodeType, const char *, KPGConnection *,	const QString &, bool = true);
	
	// Run simple 2-steps for dropping Table child object
	bool dropTableObjectWizard(KPGTreeItem *, const char *, KPGConnection *, const QString &, const QString &, bool = true);

/////////////////////////////////////////////////////////////////////
//                           SIGNALS
/////////////////////////////////////////////////////////////////////
signals:

	/** emitted, when an tree item is selected */
	void sigSelectionChanged( const KPGTreeItem::eNodeType, KPGTreeItem *, unsigned int, unsigned int);

	// Request for enabling Run Query action
  void sigEnableRunQueryAction(bool);
	
	// Request for enabling File Open, Save and Export action
	void sigEnableFileActions(bool, bool, bool);
	
	// Request for enabling Edit actions
	void sigEnableEditActions(bool, bool, bool, bool, bool, bool);
	
	/** Emitted, when a context menu is requested */
	void sigContextMenuRequested(const QString & szMenuName, const QPoint & pos );
};

#endif // KPOGREVIEW_H
